@@ -132,15 +132,9 @@ void KoCanvasControllerWidget::Private::emitPointerPositionChangedSignals(QEvent
 
 void KoCanvasControllerWidget::Private::activate()
 {
-    QWidget *parent = q;
-    while (parent->parentWidget()) {
-        parent = parent->parentWidget();
-    }
-    KoCanvasSupervisor *observerProvider = dynamic_cast<KoCanvasSupervisor*>(parent);
     if (!observerProvider) {
         return;
     }
-
     KoCanvasBase *canvas = q->canvas();
     Q_FOREACH (KoCanvasObserverBase *docker, observerProvider->canvasObservers()) {
         KoCanvasObserverBase *observer = dynamic_cast<KoCanvasObserverBase*>(docker);
@@ -153,11 +147,6 @@ void KoCanvasControllerWidget::Private::activate()
 
 void KoCanvasControllerWidget::Private::unsetCanvas()
 {
-    QWidget *parent = q;
-    while (parent->parentWidget()) {
-        parent = parent->parentWidget();
-    }
-    KoCanvasSupervisor *observerProvider = dynamic_cast<KoCanvasSupervisor*>(parent);
     if (!observerProvider) {
         return;
     }
@@ -172,10 +161,10 @@ void KoCanvasControllerWidget::Private::unsetCanvas()
 }
 
 ////////////
-KoCanvasControllerWidget::KoCanvasControllerWidget(KActionCollection * actionCollection, QWidget *parent)
+KoCanvasControllerWidget::KoCanvasControllerWidget(KActionCollection * actionCollection, KoCanvasSupervisor *observerProvider, QWidget *parent)
     : QAbstractScrollArea(parent)
     , KoCanvasController(actionCollection)
-    , d(new Private(this))
+    , d(new Private(this, observerProvider))
 {
     // We need to set this as QDeclarativeView sets them a bit different from QAbstractScrollArea
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
@@ -583,6 +572,17 @@ bool KoCanvasControllerWidget::focusNextPrevChild(bool)
     return false;
 }
 
+bool KoCanvasControllerWidget::viewportEvent(QEvent *event) {
+    // Workaround: Don't let QAbstractScrollArea handle Gesture events. Because
+    // Qt's detection of touch point positions is a bit buggy, it is handled
+    // with custom algorithms in the KisInputManager. But we must also not let
+    // the corresponding event propagate to the parent QAbstractScrollArea.
+    if (event->type() == QEvent::Gesture) {
+        return false;
+    }
+    return QAbstractScrollArea::viewportEvent(event);
+}
+
 void KoCanvasControllerWidget::setMargin(int margin)
 {
     KoCanvasController::setMargin(margin);
