@@ -49,12 +49,23 @@
 #include <QApplication>
 
 #include <QPainter>
+#include <QPainterPath>
 #include <QTimer>
 #include <FlakeDebug.h>
 
 #include "kis_painting_tweaks.h"
+#include "kis_debug.h"
+#include "KisForest.h"
+#include <unordered_set>
 
-bool KoShapeManager::Private::shapeUsedInRenderingTree(KoShape *shape)
+
+namespace {
+
+/**
+ * Returns whether the shape should be added to the RTree for collision and ROI
+ * detection.
+ */
+inline bool shapeUsedInRenderingTree(KoShape *shape)
 {
     // FIXME: make more general!
 
@@ -63,6 +74,193 @@ bool KoShapeManager::Private::shapeUsedInRenderingTree(KoShape *shape)
             !(dynamic_cast<KoSvgTextChunkShape*>(shape) && !dynamic_cast<KoSvgTextShape*>(shape));
 }
 
+/**
+ * Returns whether a shape should be added to the rendering tree because of
+ * its clip mask/path or effects.
+ */
+inline bool shapeHasGroupEffects(KoShape *shape) {
+    return shape->clipPath() ||
+        (shape->filterEffectStack() && !shape->filterEffectStack()->isEmpty()) ||
+        shape->clipMask();
+}
+
+/**
+ * Returns true if the shape is not fully transparent
+ */
+inline bool shapeIsVisible(KoShape *shape) {
+    return shape->isVisible(false) && shape->transparency() < 1.0;
+}
+
+
+/**
+ * Populate \p tree with the subtree of shapes pointed by a shape \p parentShape.
+ * All new shapes are added as children of \p parentIt. Please take it into account
+ * that \c *parentIt might be not the same as \c parentShape, because \c parentShape
+ * may be hidden from rendering.
+ */
+void populateRenderSubtree(KoShape *parentShape,
+                           KisForest<KoShape*>::child_iterator parentIt,
+                           KisForest<KoShape*> &tree,
+                           std::function<bool(KoShape*)> shouldIncludeNode,
+                           std::function<bool(KoShape*)> shouldEnterSubtree)
+{
+    KoShapeContainer *parentContainer = dynamic_cast<KoShapeContainer*>(parentShape);
+    if (!parentContainer) return;
+
+    QList<KoShape*> children = parentContainer->shapes();
+    std::sort(children.begin(), children.end(), KoShape::compareShapeZIndex);
+
+    for (auto it = children.constBegin(); it != children.constEnd(); ++it) {
+        auto newParentIt = parentIt;
+
+        if (shouldIncludeNode(*it)) {
+            newParentIt = tree.insert(childEnd(parentIt), *it);
+        }
+
+        if (shouldEnterSubtree(*it)) {
+            populateRenderSubtree(*it, newParentIt, tree, shouldIncludeNode, shouldEnterSubtree);
+        }
+    }
+
+}
+
+/**
+ * Build a rendering tree for **leaf** nodes defined by \p leafNodes
+ *
+ * Sometimes we should render only a part of the layer (e.g. when we render
+ * in patches). So we shouldn't render the whole graph. The problem is that
+ * some of the shapes may have parents with clip paths/masks and/or effects.
+ * In such a case, these parents should be also included into the rendering
+ * process.
+ *
+ * \c buildRenderTree() builds a graph for such rendering. It includes the
+ * leaf shapes themselves, and all parent shapes that have some effects affecting
+ * these shapes.
+ */
+void buildRenderTree(QList<KoShape*> leafShapes,
+                     KisForest<KoShape*> &tree)
+{
+    QList<KoShape*> sortedShapes = leafShapes;
+    std::sort(sortedShapes.begin(), sortedShapes.end(), KoShape::compareShapeZIndex);
+
+    std::unordered_set<KoShape*> includedShapes;
+
+    Q_FOREACH (KoShape *shape, sortedShapes) {
+        bool shouldSkipShape = !shapeIsVisible(shape);
+        if (shouldSkipShape) continue;
+
+        bool shapeIsPartOfIncludedSubtree = false;
+        QVector<KoShape*> hierarchy = {shape};
+
+        while ((shape = shape->parent())) {
+            if (!shapeIsVisible(shape)) {
+                shouldSkipShape = true;
+                break;
+            }
+
+            if (includedShapes.find(shape) != end(includedShapes)) {
+                shapeIsPartOfIncludedSubtree = true;
+                break;
+            }
+
+            if (shapeHasGroupEffects(shape)) {
+                hierarchy << shape;
+            }
+        }
+
+        if (shouldSkipShape) continue;
+
+        if (!shapeIsPartOfIncludedSubtree &&
+            includedShapes.find(hierarchy.last()) == end(includedShapes)) {
+
+            tree.insert(childEnd(tree), hierarchy.last());
+        }
+        std::copy(hierarchy.begin(), hierarchy.end(),
+                  std::inserter(includedShapes, end(includedShapes)));
+    }
+
+    auto shouldIncludeShape =
+        [includedShapes] (KoShape *shape) {
+            // included shapes are guaranteed to be visible
+            return includedShapes.find(shape) != end(includedShapes);
+        };
+
+    for (auto it = childBegin(tree); it != childEnd(tree); ++it) {
+        populateRenderSubtree(*it, it, tree, shouldIncludeShape, &shapeIsVisible);
+    }
+}
+
+/**
+ * Render the prebuilt rendering tree on \p painter
+ */
+void renderShapes(typename KisForest<KoShape*>::child_iterator beginIt,
+                  typename KisForest<KoShape*>::child_iterator endIt,
+                  QPainter &painter,
+                  KoShapePaintingContext &paintContext)
+{
+    for (auto it = beginIt; it != endIt; ++it) {
+        KoShape *shape = *it;
+
+        KisQPainterStateSaver saver(&painter);
+
+        if (!isEnd(parent(it))) {
+            painter.setTransform(shape->transformation() * painter.transform());
+        } else {
+            painter.setTransform(shape->absoluteTransformation() * painter.transform());
+        }
+
+        KoClipPath::applyClipping(shape, painter);
+
+        qreal transparency = shape->transparency(true);
+        if (transparency > 0.0) {
+            painter.setOpacity(1.0-transparency);
+        }
+
+        if (shape->shadow()) {
+            KisQPainterStateSaver saver(&painter);
+            shape->shadow()->paint(shape, painter);
+        }
+
+        QScopedPointer<KoClipMaskPainter> clipMaskPainter;
+        QPainter *shapePainter = &painter;
+
+        KoClipMask *clipMask = shape->clipMask();
+        if (clipMask) {
+            const QRectF bounds = painter.transform().mapRect(shape->outlineRect());
+
+            clipMaskPainter.reset(new KoClipMaskPainter(&painter, bounds/*shape->boundingRect())*/));
+            shapePainter = clipMaskPainter->shapePainter();
+        }
+
+        /**
+         * We expect the shape to save/restore the painter's state itself. Such design was not
+         * not always here, so we need a period of sanity checks to ensure all the shapes are
+         * ported correctly.
+         */
+        const QTransform sanityCheckTransformSaved = shapePainter->transform();
+
+        renderShapes(childBegin(it), childEnd(it), *shapePainter, paintContext);
+
+        shape->paint(*shapePainter, paintContext);
+        shape->paintStroke(*shapePainter, paintContext);
+
+        KIS_SAFE_ASSERT_RECOVER(shapePainter->transform() == sanityCheckTransformSaved) {
+            shapePainter->setTransform(sanityCheckTransformSaved);
+        }
+
+        if (clipMask) {
+            clipMaskPainter->maskPainter()->save();
+
+            shape->clipMask()->drawMask(clipMaskPainter->maskPainter(), shape);
+            clipMaskPainter->renderOnGlobalPainter();
+
+            clipMaskPainter->maskPainter()->restore();
+        }
+    }
+}
+
+}
+
 void KoShapeManager::Private::updateTree()
 {
     bool selectionModified = false;
@@ -125,25 +323,6 @@ void KoShapeManager::Private::forwardCompressedUdpate()
 
 }
 
-void KoShapeManager::Private::paintGroup(KoShapeGroup *group, QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
-{
-    QList<KoShape*> shapes = group->shapes();
-    std::sort(shapes.begin(), shapes.end(), KoShape::compareShapeZIndex);
-    Q_FOREACH (KoShape *child, shapes) {
-        // we paint recursively here, so we do not have to check recursively for visibility
-        if (!child->isVisible(false))
-            continue;
-        KoShapeGroup *childGroup = dynamic_cast<KoShapeGroup*>(child);
-        if (childGroup) {
-            paintGroup(childGroup, painter, converter, paintContext);
-        } else {
-            painter.save();
-            KoShapeManager::renderSingleShape(child, painter, converter, paintContext);
-            painter.restore();
-        }
-    }
-}
-
 KoShapeManager::KoShapeManager(KoCanvasBase *canvas, const QList<KoShape *> &shapes)
     : d(new Private(this, canvas))
 {
@@ -174,7 +353,7 @@ KoShapeManager::KoShapeManager(KoCanvasBase *canvas)
 void KoShapeManager::Private::unlinkFromShapesRecursively(const QList<KoShape*> &shapes)
 {
     Q_FOREACH (KoShape *shape, shapes) {
-        shape->priv()->removeShapeManager(q);
+        shape->removeShapeManager(q);
 
         KoShapeContainer *container = dynamic_cast<KoShapeContainer*>(shape);
         if (container) {
@@ -220,10 +399,10 @@ void KoShapeManager::addShape(KoShape *shape, Repaint repaint)
 
         if (d->shapes.contains(shape))
             return;
-        shape->priv()->addShapeManager(this);
+        shape->addShapeManager(this);
         d->shapes.append(shape);
 
-        if (d->shapeUsedInRenderingTree(shape)) {
+        if (shapeUsedInRenderingTree(shape)) {
             QMutexLocker l2(&d->treeMutex);
 
             QRectF br(shape->boundingRect());
@@ -253,13 +432,13 @@ void KoShapeManager::remove(KoShape *shape)
         QMutexLocker l2(&d->treeMutex);
 
         dirtyRect = shape->absoluteOutlineRect();
-        shape->priv()->removeShapeManager(this);
 
+        shape->removeShapeManager(this);
         d->selection->deselect(shape);
         d->aggregate4update.remove(shape);
         d->compressedUpdatedShapes.remove(shape);
 
-        if (d->shapeUsedInRenderingTree(shape)) {
+        if (shapeUsedInRenderingTree(shape)) {
             d->tree.remove(shape);
         }
         d->shapes.removeAll(shape);
@@ -307,8 +486,85 @@ KoShapeManager::ShapeInterface *KoShapeManager::shapeInterface()
     return &d->shapeInterface;
 }
 
+void KoShapeManager::preparePaintJobs(PaintJobsOrder &jobsOrder,
+                                      KoShape *excludeRoot)
+{
+    d->updateTree();
 
-void KoShapeManager::paint(QPainter &painter, const KoViewConverter &converter, bool forPrint)
+    QMutexLocker l1(&d->shapesMutex);
+
+    QSet<KoShape*> rootShapesSet;
+    Q_FOREACH (KoShape *shape, d->shapes) {
+        while (shape->parent() && shape->parent() != excludeRoot) {
+            shape = shape->parent();
+        }
+
+        if (!rootShapesSet.contains(shape) && shape != excludeRoot) {
+            rootShapesSet.insert(shape);
+        }
+    }
+
+    const QList<KoShape*> rootShapes = rootShapesSet.toList();
+
+    QList<KoShape*> newRootShapes;
+
+    Q_FOREACH (KoShape *srcShape, rootShapes) {
+        KIS_SAFE_ASSERT_RECOVER(srcShape->parent() == excludeRoot) { continue; }
+
+        KoShape *clonedShape = srcShape->cloneShape();
+
+        KoShapeContainer *parentShape = srcShape->parent();
+
+        if (parentShape && !parentShape->transformation().isIdentity()) {
+            clonedShape->applyAbsoluteTransformation(parentShape->transformation());
+        }
+
+        newRootShapes << clonedShape;
+    }
+
+    PaintJobsOrder result;
+
+    PaintJob::SharedSafeStorage shapesStorage = std::make_shared<PaintJob::ShapesStorage>();
+    Q_FOREACH (KoShape *shape, newRootShapes) {
+        shapesStorage->emplace_back(std::unique_ptr<KoShape>(shape));
+    }
+
+    const QList<KoShape*> originalShapes = KoShape::linearizeSubtreeSorted(rootShapes);
+    const QList<KoShape*> clonedShapes = KoShape::linearizeSubtreeSorted(newRootShapes);
+    KIS_SAFE_ASSERT_RECOVER_RETURN(clonedShapes.size() == originalShapes.size());
+
+    QHash<KoShape*, KoShape*> clonedFromOriginal;
+    for (int i = 0; i < originalShapes.size(); i++) {
+        clonedFromOriginal[originalShapes[i]] = clonedShapes[i];
+    }
+
+
+    for (auto it = std::begin(jobsOrder.jobs); it != std::end(jobsOrder.jobs); ++it) {
+        QMutexLocker l(&d->treeMutex);
+        QList<KoShape*> unsortedOriginalShapes = d->tree.intersects(it->docUpdateRect);
+
+        it->allClonedShapes = shapesStorage;
+
+        Q_FOREACH (KoShape *shape, unsortedOriginalShapes) {
+            KIS_SAFE_ASSERT_RECOVER(shapeUsedInRenderingTree(shape)) { continue; }
+            it->shapes << clonedFromOriginal[shape];
+        }
+    }
+}
+
+void KoShapeManager::paintJob(QPainter &painter, const KoShapeManager::PaintJob &job, bool forPrint)
+{
+    painter.setPen(Qt::NoPen);  // painters by default have a black stroke, lets turn that off.
+    painter.setBrush(Qt::NoBrush);
+
+    KisForest<KoShape*> renderTree;
+    buildRenderTree(job.shapes, renderTree);
+
+    KoShapePaintingContext paintContext(d->canvas, forPrint); //FIXME
+    renderShapes(childBegin(renderTree), childEnd(renderTree), painter, paintContext);
+}
+
+void KoShapeManager::paint(QPainter &painter, bool forPrint)
 {
     d->updateTree();
 
@@ -321,228 +577,29 @@ void KoShapeManager::paint(QPainter &painter, const KoViewConverter &converter,
     if (painter.hasClipping()) {
         QMutexLocker l(&d->treeMutex);
 
-        QRectF rect = converter.viewToDocument(KisPaintingTweaks::safeClipBoundingRect(painter));
+        QRectF rect = KisPaintingTweaks::safeClipBoundingRect(painter);
         unsortedShapes = d->tree.intersects(rect);
     } else {
         unsortedShapes = d->shapes;
         warnFlake << "KoShapeManager::paint  Painting with a painter that has no clipping will lead to too much being painted!";
     }
 
-    // filter all hidden shapes from the list
-    // also filter shapes with a parent which has filter effects applied
-    QList<KoShape*> sortedShapes;
-    foreach (KoShape *shape, unsortedShapes) {
-        if (!shape->isVisible())
-            continue;
-        bool addShapeToList = true;
-        // check if one of the shapes ancestors have filter effects
-        KoShapeContainer *parent = shape->parent();
-        while (parent) {
-            // parent must be part of the shape manager to be taken into account
-            if (!d->shapes.contains(parent))
-                break;
-            if (parent->filterEffectStack() && !parent->filterEffectStack()->isEmpty()) {
-                addShapeToList = false;
-                break;
-            }
-            parent = parent->parent();
-        }
-        if (addShapeToList) {
-            sortedShapes.append(shape);
-        } else if (parent) {
-            sortedShapes.append(parent);
-        }
-    }
-
-    std::sort(sortedShapes.begin(), sortedShapes.end(), KoShape::compareShapeZIndex);
-
     KoShapePaintingContext paintContext(d->canvas, forPrint); //FIXME
 
-    foreach (KoShape *shape, sortedShapes) {
-        renderSingleShape(shape, painter, converter, paintContext);
-    }
-
-#ifdef CALLIGRA_RTREE_DEBUG
-    // paint tree
-    qreal zx = 0;
-    qreal zy = 0;
-    converter.zoom(&zx, &zy);
-    painter.save();
-    painter.scale(zx, zy);
-    d->tree.paint(painter);
-    painter.restore();
-#endif
-
-    if (! forPrint) {
-        KoShapePaintingContext paintContext(d->canvas, forPrint); //FIXME
-        d->selection->paint(painter, converter, paintContext);
-    }
+    KisForest<KoShape*> renderTree;
+    buildRenderTree(unsortedShapes, renderTree);
+    renderShapes(childBegin(renderTree), childEnd(renderTree), painter, paintContext);
 }
 
-void KoShapeManager::renderSingleShape(KoShape *shape, QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
+void KoShapeManager::renderSingleShape(KoShape *shape, QPainter &painter, KoShapePaintingContext &paintContext)
 {
-    KisQPainterStateSaver saver(&painter);
+    KisForest<KoShape*> renderTree;
 
-    // apply shape clipping
-    KoClipPath::applyClipping(shape, painter, converter);
+    KoViewConverter converter;
 
-    // apply transformation
-    painter.setTransform(shape->absoluteTransformation(&converter) * painter.transform());
-
-    // paint the shape
-    paintShape(shape, painter, converter, paintContext);
-}
-
-void KoShapeManager::paintShape(KoShape *shape, QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
-{
-    qreal transparency = shape->transparency(true);
-    if (transparency > 0.0) {
-        painter.setOpacity(1.0-transparency);
-    }
-
-    if (shape->shadow()) {
-        painter.save();
-        shape->shadow()->paint(shape, painter, converter);
-        painter.restore();
-    }
-    if (!shape->filterEffectStack() || shape->filterEffectStack()->isEmpty()) {
-
-        QScopedPointer<KoClipMaskPainter> clipMaskPainter;
-        QPainter *shapePainter = &painter;
-
-        KoClipMask *clipMask = shape->clipMask();
-        if (clipMask) {
-            clipMaskPainter.reset(new KoClipMaskPainter(&painter, shape->boundingRect()));
-            shapePainter = clipMaskPainter->shapePainter();
-        }
-
-        /**
-         * We expect the shape to save/restore the painter's state itself. Such design was not
-         * not always here, so we need a period of sanity checks to ensure all the shapes are
-         * ported correctly.
-         */
-        const QTransform sanityCheckTransformSaved = shapePainter->transform();
-
-        shape->paint(*shapePainter, converter, paintContext);
-        shape->paintStroke(*shapePainter, converter, paintContext);
-
-        KIS_SAFE_ASSERT_RECOVER(shapePainter->transform() == sanityCheckTransformSaved) {
-            shapePainter->setTransform(sanityCheckTransformSaved);
-        }
-
-        if (clipMask) {
-            shape->clipMask()->drawMask(clipMaskPainter->maskPainter(), shape);
-            clipMaskPainter->renderOnGlobalPainter();
-        }
-
-    } else {
-        // TODO: clipping mask is not implemented for this case!
-
-        // There are filter effects, then we need to prerender the shape on an image, to filter it
-        QRectF shapeBound(QPointF(), shape->size());
-        // First step, compute the rectangle used for the image
-        QRectF clipRegion = shape->filterEffectStack()->clipRectForBoundingRect(shapeBound);
-        // convert clip region to view coordinates
-        QRectF zoomedClipRegion = converter.documentToView(clipRegion);
-        // determine the offset of the clipping rect from the shapes origin
-        QPointF clippingOffset = zoomedClipRegion.topLeft();
-
-        // Initialize the buffer image
-        QImage sourceGraphic(zoomedClipRegion.size().toSize(), QImage::Format_ARGB32_Premultiplied);
-        sourceGraphic.fill(qRgba(0,0,0,0));
-
-        QHash<QString, QImage> imageBuffers;
-
-        QSet<QString> requiredStdInputs = shape->filterEffectStack()->requiredStandarsInputs();
-
-        if (requiredStdInputs.contains("SourceGraphic") || requiredStdInputs.contains("SourceAlpha")) {
-            // Init the buffer painter
-            QPainter imagePainter(&sourceGraphic);
-            imagePainter.translate(-1.0f*clippingOffset);
-            imagePainter.setPen(Qt::NoPen);
-            imagePainter.setBrush(Qt::NoBrush);
-            imagePainter.setRenderHint(QPainter::Antialiasing, painter.testRenderHint(QPainter::Antialiasing));
-
-            // Paint the shape on the image
-            KoShapeGroup *group = dynamic_cast<KoShapeGroup*>(shape);
-            if (group) {
-                // the childrens matrix contains the groups matrix as well
-                // so we have to compensate for that before painting the children
-                imagePainter.setTransform(group->absoluteTransformation(&converter).inverted(), true);
-                Private::paintGroup(group, imagePainter, converter, paintContext);
-            } else {
-                imagePainter.save();
-                shape->paint(imagePainter, converter, paintContext);
-                shape->paintStroke(imagePainter, converter, paintContext);
-                imagePainter.restore();
-                imagePainter.end();
-            }
-        }
-        if (requiredStdInputs.contains("SourceAlpha")) {
-            QImage sourceAlpha = sourceGraphic;
-            sourceAlpha.fill(qRgba(0,0,0,255));
-            sourceAlpha.setAlphaChannel(sourceGraphic.alphaChannel());
-            imageBuffers.insert("SourceAlpha", sourceAlpha);
-        }
-        if (requiredStdInputs.contains("FillPaint")) {
-            QImage fillPaint = sourceGraphic;
-            if (shape->background()) {
-                QPainter fillPainter(&fillPaint);
-                QPainterPath fillPath;
-                fillPath.addRect(fillPaint.rect().adjusted(-1,-1,1,1));
-                shape->background()->paint(fillPainter, converter, paintContext, fillPath);
-            } else {
-                fillPaint.fill(qRgba(0,0,0,0));
-            }
-            imageBuffers.insert("FillPaint", fillPaint);
-        }
-
-        imageBuffers.insert("SourceGraphic", sourceGraphic);
-        imageBuffers.insert(QString(), sourceGraphic);
-
-        KoFilterEffectRenderContext renderContext(converter);
-        renderContext.setShapeBoundingBox(shapeBound);
-
-        QImage result;
-        QList<KoFilterEffect*> filterEffects = shape->filterEffectStack()->filterEffects();
-        // Filter
-        foreach (KoFilterEffect *filterEffect, filterEffects) {
-            QRectF filterRegion = filterEffect->filterRectForBoundingRect(shapeBound);
-            filterRegion = converter.documentToView(filterRegion);
-            QRect subRegion = filterRegion.translated(-clippingOffset).toRect();
-            // set current filter region
-            renderContext.setFilterRegion(subRegion & sourceGraphic.rect());
-
-            if (filterEffect->maximalInputCount() <= 1) {
-                QList<QString> inputs = filterEffect->inputs();
-                QString input = inputs.count() ? inputs.first() : QString();
-                // get input image from image buffers and apply the filter effect
-                QImage image = imageBuffers.value(input);
-                if (!image.isNull()) {
-                    result = filterEffect->processImage(imageBuffers.value(input), renderContext);
-                }
-            } else {
-                QList<QImage> inputImages;
-                Q_FOREACH (const QString &input, filterEffect->inputs()) {
-                    QImage image = imageBuffers.value(input);
-                    if (!image.isNull())
-                        inputImages.append(imageBuffers.value(input));
-                }
-                // apply the filter effect
-                if (filterEffect->inputs().count() == inputImages.count())
-                    result = filterEffect->processImages(inputImages, renderContext);
-            }
-            // store result of effect
-            imageBuffers.insert(filterEffect->output(), result);
-        }
-
-        KoFilterEffect *lastEffect = filterEffects.last();
-
-        // Paint the result
-        painter.save();
-        painter.drawImage(clippingOffset, imageBuffers.value(lastEffect->output()));
-        painter.restore();
-    }
+    auto root = renderTree.insert(childBegin(renderTree), shape);
+    populateRenderSubtree(shape, root, renderTree, &shapeIsVisible, &shapeIsVisible);
+    renderShapes(childBegin(renderTree), childEnd(renderTree), painter, paintContext);
 }
 
 KoShape *KoShapeManager::shapeAt(const QPointF &position, KoFlake::ShapeSelection selection, bool omitHiddenShapes)
@@ -623,7 +680,7 @@ QList<KoShape *> KoShapeManager::shapesAt(const QRectF &rect, bool omitHiddenSha
         if (omitHiddenShapes && !shape->isVisible()) {
             shapes.removeAt(count);
         } else {
-            const QPainterPath outline = shape->absoluteTransformation(0).map(shape->outline());
+            const QPainterPath outline = shape->absoluteTransformation().map(shape->outline());
 
             if (!containedMode && !outline.intersects(rect) && !outline.contains(rect)) {
                 shapes.removeAt(count);
