@@ -66,6 +66,7 @@
 #include <FlakeDebug.h>
 #include <klocalizedstring.h>
 #include <QPainter>
+#include <QPainterPath>
 #include <QBitmap>
 #include <QTabWidget>
 
@@ -478,7 +479,7 @@ void KoPathTool::paint(QPainter &painter, const KoViewConverter &converter)
 
     Q_FOREACH (KoPathShape *shape, m_pointSelection.selectedShapes()) {
         KisHandlePainterHelper helper =
-                KoShape::createHandlePainterHelper(&painter, shape, converter, m_handleRadius);
+                KoShape::createHandlePainterHelperView(&painter, shape, converter, m_handleRadius);
         helper.setHandleStyle(KisHandleStyle::primarySelection());
 
         KoParameterShape * parameterShape = dynamic_cast<KoParameterShape*>(shape);
@@ -521,7 +522,7 @@ void KoPathTool::paint(QPainter &painter, const KoViewConverter &converter)
             KIS_SAFE_ASSERT_RECOVER_RETURN(segment.isValid());
 
             KisHandlePainterHelper helper =
-                    KoShape::createHandlePainterHelper(&painter, shape, converter, m_handleRadius);
+                    KoShape::createHandlePainterHelperView(&painter, shape, converter, m_handleRadius);
             helper.setHandleStyle(KisHandleStyle::secondarySelection());
 
             QPainterPath path;
@@ -538,7 +539,7 @@ void KoPathTool::paint(QPainter &painter, const KoViewConverter &converter)
 
     if (m_currentStrategy) {
         painter.save();
-        KoShape::applyConversion(painter, converter);
+        painter.setTransform(converter.documentToView(), true);
         d->canvas->snapGuide()->paint(painter, converter);
         painter.restore();
     }
@@ -799,13 +800,13 @@ void KoPathTool::keyPressEvent(QKeyEvent *event)
     } else {
         switch (event->key()) {
 #ifndef NDEBUG
-        case Qt::Key_D:
-            if (m_pointSelection.objectCount() == 1) {
-                QList<KoPathPointData> selectedPoints = m_pointSelection.selectedPointsData();
-                KoPathShapePrivate *p = static_cast<KoPathShapePrivate*>(selectedPoints[0].pathShape->priv());
-                p->debugPath();
-            }
-            break;
+//        case Qt::Key_D:
+//            if (m_pointSelection.objectCount() == 1) {
+//                QList<KoPathPointData> selectedPoints = m_pointSelection.selectedPointsData();
+//                KoPathShapePrivate *p = static_cast<KoPathShapePrivate*>(selectedPoints[0].pathShape->priv());
+//                p->debugPath();
+//            }
+//            break;
 #endif
         case Qt::Key_B:
             if (m_pointSelection.size() == 1)
@@ -1181,7 +1182,7 @@ void KoPathTool::documentResourceChanged(int key, const QVariant & res)
         // repaint with the bigger of old and new handle radius
         int maxRadius = qMax(m_handleRadius, oldHandleRadius);
         Q_FOREACH (KoPathShape *shape, m_pointSelection.selectedShapes()) {
-            QRectF controlPointRect = shape->absoluteTransformation(0).map(shape->outline()).controlPointRect();
+            QRectF controlPointRect = shape->absoluteTransformation().map(shape->outline()).controlPointRect();
             repaint(controlPointRect.adjusted(-maxRadius, -maxRadius, maxRadius, maxRadius));
         }
     }
