@@ -107,16 +107,16 @@ struct KisSuspendProjectionUpdatesStrokeStrategy::Private
             for (; it != end; ++it) {
                 KisNodeSP node = it.key();
 
-                QRegion region;
+                QVector<QRect> dirtyRects;
 
                 bool resetAnimationCache = false;
                 Q_FOREACH (const Request &req, it.value()) {
-                    region += alignRect(req.rect, step);
+                    dirtyRects += alignRect(req.rect, step);
                     resetAnimationCache |= req.resetAnimationCache;
                 }
 
                 // FIXME: constness: port rPU to SP
-                listener->requestProjectionUpdate(const_cast<KisNode*>(node.data()), region.rects(), resetAnimationCache);
+                listener->requestProjectionUpdate(const_cast<KisNode*>(node.data()), KisRegion(dirtyRects).rects(), resetAnimationCache);
             }
         }
 
@@ -365,7 +365,9 @@ struct KisSuspendProjectionUpdatesStrokeStrategy::Private
 };
 
 KisSuspendProjectionUpdatesStrokeStrategy::KisSuspendProjectionUpdatesStrokeStrategy(KisImageWSP image, bool suspend, SharedDataSP sharedData)
-    : KisRunnableBasedStrokeStrategy(suspend ? "suspend_stroke_strategy" : "resume_stroke_strategy"),
+    : KisRunnableBasedStrokeStrategy(suspend ?
+                                     QLatin1String("suspend_stroke_strategy") :
+                                     QLatin1String("resume_stroke_strategy")),
       m_d(new Private)
 {
     m_d->image = image;
