@@ -98,8 +98,8 @@ struct Q_DECL_HIDDEN KisTransformMask::Private
 };
 
 
-KisTransformMask::KisTransformMask()
-    : KisEffectMask(),
+KisTransformMask::KisTransformMask(const QString &name)
+    : KisEffectMask(name),
       m_d(new Private())
 {
     setTransformParams(
@@ -374,6 +374,13 @@ QRect KisTransformMask::needRect(const QRect& rect, PositionToFilthy pos) const
         KisSafeTransform transform(m_d->worker.forwardTransform(), limitingRect, interestRect);
         needRect = transform.mapRectBackward(rect);
 
+        /**
+         * When sampling affine transformations we use KisRandomSubAccessor,
+         * which uses bilinear interpolation for calculating pixels. Therefore,
+         * we need to extend the sides of the need rect by one pixel.
+         */
+        needRect = kisGrowRect(needRect, 1);
+
     } else {
         needRect = m_d->params->nonAffineNeedRect(rect, interestRect);
     }
@@ -482,7 +489,7 @@ KisKeyframeChannel *KisTransformMask::requestKeyframeChannel(const QString &id)
             animatedParams = dynamic_cast<KisAnimatedTransformParamsInterface*>(converted.data());
         }
 
-        KisKeyframeChannel *channel = animatedParams->getKeyframeChannel(id, parent()->original()->defaultBounds());
+        KisKeyframeChannel *channel = animatedParams->getKeyframeChannel(id, parent());
         if (channel) return channel;
     }
 
