@@ -53,6 +53,7 @@
 #include <kis_coordinates_converter.h>
 #include <kis_time_range.h>
 #include <KisImportExportErrorCode.h>
+#include <KisPart.h>
 
 #include <KoColor.h>
 #include <KoColorSpace.h>
@@ -76,17 +77,23 @@
 struct Document::Private {
     Private() {}
     QPointer<KisDocument> document;
+    bool ownsDocument {false};
 };
 
-Document::Document(KisDocument *document, QObject *parent)
+Document::Document(KisDocument *document, bool ownsDocument, QObject *parent)
     : QObject(parent)
     , d(new Private)
 {
     d->document = document;
+    d->ownsDocument = ownsDocument;
 }
 
 Document::~Document()
 {
+    if (d->ownsDocument && d->document) {
+        KisPart::instance()->removeDocument(d->document);
+        delete d->document;
+    }
     delete d;
 }
 
@@ -157,7 +164,8 @@ Node *Document::nodeByName(const QString &name) const
 {
     if (!d->document) return 0;
     KisNodeSP node = d->document->image()->rootLayer()->findChildByName(name);
-    return new Node(d->document->image(), node);
+    if (node.isNull()) return 0;
+    return Node::createNode(d->document->image(), node);
 }
 
 
@@ -317,7 +325,7 @@ Node *Document::rootNode() const
     KisImageSP image = d->document->image();
     if (!image) return 0;
 
-    return new Node(image, image->root());
+    return Node::createNode(image, image->root());
 }
 
 Selection *Document::selection() const
@@ -444,6 +452,7 @@ QByteArray Document::pixelData(int x, int y, int w, int h) const
 bool Document::close()
 {
     bool retval = d->document->closeUrl(false);
+
     Q_FOREACH(KisView *view, KisPart::instance()->views()) {
         if (view->document() == d->document) {
             view->close();
@@ -452,7 +461,11 @@ bool Document::close()
         }
     }
 
-    KisPart::instance()->removeDocument(d->document);
+    KisPart::instance()->removeDocument(d->document, !d->ownsDocument);
+    if (d->ownsDocument) {
+        delete d->document;
+    }
+
     d->document = 0;
     return retval;
 }
@@ -464,6 +477,7 @@ void Document::crop(int x, int y, int w, int h)
     if (!image) return;
     QRect rc(x, y, w, h);
     image->cropImage(rc);
+    image->waitForDone();
 }
 
 bool Document::exportImage(const QString &filename, const InfoObject &exportConfiguration)
@@ -481,6 +495,7 @@ void Document::flatten()
     if (!d->document) return;
     if (!d->document->image()) return;
     d->document->image()->flatten(0);
+    d->document->image()->waitForDone();
 }
 
 void Document::resizeImage(int x, int y, int w, int h)
@@ -495,6 +510,7 @@ void Document::resizeImage(int x, int y, int w, int h)
     rc.setHeight(h);
 
     image->resizeImage(rc);
+    image->waitForDone();
 }
 
 void Document::scaleImage(int w, int h, int xres, int yres, QString strategy)
@@ -510,6 +526,7 @@ void Document::scaleImage(int w, int h, int xres, int yres, QString strategy)
     if (!actualStrategy) actualStrategy = KisFilterStrategyRegistry::instance()->get("Bicubic");
 
     image->scaleImage(rc.size(), xres/72, yres/72, actualStrategy);
+    image->waitForDone();
 }
 
 void Document::rotateImage(double radians)
@@ -518,6 +535,7 @@ void Document::rotateImage(double radians)
     KisImageSP image = d->document->image();
     if (!image) return;
     image->rotateImage(radians);
+    image->waitForDone();
 }
 
 void Document::shearImage(double angleX, double angleY)
@@ -526,6 +544,7 @@ void Document::shearImage(double angleX, double angleY)
     KisImageSP image = d->document->image();
     if (!image) return;
     image->shear(angleX, angleY);
+    image->waitForDone();
 }
 
 bool Document::save()
@@ -584,17 +603,18 @@ Node* Document::createNode(const QString &name, const QString &nodeType)
         node = new Node(image, new KisShapeLayer(d->document->shapeController(), image, name, OPACITY_OPAQUE_U8));
     }
     else if (nodeType.toLower()  == "transparencymask") {
-        node = new Node(image, new KisTransparencyMask());
+        node = new Node(image, new KisTransparencyMask(name));
     }
     else if (nodeType.toLower()  == "filtermask") {
-        node = new Node(image, new KisFilterMask());
+        node = new Node(image, new KisFilterMask(name));
     }
     else if (nodeType.toLower()  == "transformmask") {
-        node = new Node(image, new KisTransformMask());
+        node = new Node(image, new KisTransformMask(name));
     }
     else if (nodeType.toLower()  == "selectionmask") {
-        node = new Node(image, new KisSelectionMask(image));
+        node = new Node(image, new KisSelectionMask(image, name));
     }
+
     return node;
 }
 
@@ -634,7 +654,7 @@ FillLayer *Document::createFillLayer(const QString &name, const QString generato
     KisGeneratorSP generator = KisGeneratorRegistry::instance()->value(generatorName);
     if (generator) {
 
-        KisFilterConfigurationSP config = generator->defaultConfiguration();
+        KisFilterConfigurationSP config = generator->factoryConfiguration();
         Q_FOREACH(const QString property, configuration.properties().keys()) {
             config->setProperty(property, configuration.property(property));
         }
@@ -747,12 +767,6 @@ bool Document::tryBarrierLock()
     return d->document->image()->tryBarrierLock();
 }
 
-bool Document::isIdle()
-{
-    if (!d->document || !d->document->image()) return false;
-    return d->document->image()->isIdle();
-}
-
 void Document::refreshProjection()
 {
     if (!d->document || !d->document->image()) return;
@@ -803,9 +817,9 @@ Document *Document::clone() const
 {
     if (!d->document) return 0;
     QPointer<KisDocument> clone = d->document->clone();
-    Document * d = new Document(clone);
-    clone->setParent(d); // It's owned by the document, not KisPart
-    return d;
+    Document * newDocument = new Document(clone, d->ownsDocument);
+    clone->setParent(newDocument); // It's owned by the document, not KisPart
+    return newDocument;
 }
 
 void Document::setHorizontalGuides(const QList<qreal> &lines)
@@ -873,6 +887,11 @@ QPointer<KisDocument> Document::document() const
     return d->document;
 }
 
+void Document::setOwnsDocument(bool ownsDocument)
+{
+    d->ownsDocument = ownsDocument;
+}
+
 /* Animation related function */
 
 bool Document::importAnimation(const QList<QString> &files, int firstFrame, int step)
