@@ -82,6 +82,40 @@ Node::Node(KisImageSP image, KisNodeSP node, QObject *parent)
     d->node = node;
 }
 
+Node *Node::createNode(KisImageSP image, KisNodeSP node, QObject *parent)
+{
+    if (node.isNull()) {
+        return 0;
+    }
+    if (node->inherits("KisGroupLayer")) {
+        return new GroupLayer(dynamic_cast<KisGroupLayer*>(node.data()));
+    }
+    else if (node->inherits("KisCloneLayer")) {
+        return new CloneLayer(dynamic_cast<KisCloneLayer*>(node.data()));
+    }
+    else if (node->inherits("KisFileLayer")) {
+        return new FileLayer(dynamic_cast<KisFileLayer*>(node.data()));
+    }
+    else if (node->inherits("KisAdjustmentLayer")) {
+        return new FilterLayer(dynamic_cast<KisAdjustmentLayer*>(node.data()));
+    }
+    else if (node->inherits("KisGeneratorLayer")) {
+        return new FillLayer(dynamic_cast<KisGeneratorLayer*>(node.data()));
+    }
+    else if (node->inherits("KisShapeLayer")) {
+        return new VectorLayer(dynamic_cast<KisShapeLayer*>(node.data()));
+    }
+    else if (node->inherits("KisFilterMask")) {
+        return new FilterMask(image, dynamic_cast<KisFilterMask*>(node.data()));
+    }
+    else if (node->inherits("KisSelectionMask")) {
+        return new SelectionMask(image, dynamic_cast<KisSelectionMask*>(node.data()));
+    }
+    else {
+        return new Node(image, node, parent);
+    }
+}
+
 Node::~Node()
 {
     delete d;
@@ -101,7 +135,7 @@ bool Node::operator!=(const Node &other) const
 Node *Node::clone() const
 {
     KisNodeSP clone = d->node->clone();
-    Node *node = new Node(0, clone);
+    Node *node = Node::createNode(0, clone);
     return node;
 }
 
@@ -248,7 +282,6 @@ bool Node::setColorSpace(const QString &colorModel, const QString &colorDepth, c
 {
     if (!d->node) return false;
     if (!d->node->inherits("KisLayer")) return false;
-    KisLayer *layer = qobject_cast<KisLayer*>(d->node.data());
     const KoColorProfile *profile = KoColorSpaceRegistry::instance()->profileByName(colorProfile);
     const KoColorSpace *dstCs = KoColorSpaceRegistry::instance()->colorSpace(colorModel,
                                                                              colorDepth,
@@ -356,7 +389,8 @@ void Node::setOpacity(int value)
 Node* Node::parentNode() const
 {
     if (!d->node) return 0;
-    return new Node(d->image, d->node->parent());
+    if (!d->node->parent()) return 0;
+    return Node::createNode(d->image, d->node->parent());
 }
 
 QString Node::type() const
@@ -531,7 +565,7 @@ bool Node::remove()
 Node* Node::duplicate()
 {
     if (!d->node) return 0;
-    return new Node(d->image, d->node->clone());
+    return Node::createNode(d->image, d->node->clone());
 }
 
 bool Node::save(const QString &filename, double xRes, double yRes, const InfoObject &exportConfiguration, const QRect &exportRect)
@@ -575,7 +609,7 @@ Node* Node::mergeDown()
     d->image->mergeDown(qobject_cast<KisLayer*>(d->node.data()), KisMetaData::MergeStrategyRegistry::instance()->get("Drop"));
     d->image->waitForDone();
 
-    return new Node(d->image, d->node->prevSibling());
+    return Node::createNode(d->image, d->node->prevSibling());
 }
 
 void Node::scaleNode(QPointF origin, int width, int height, QString strategy)
@@ -594,6 +628,7 @@ void Node::scaleNode(QPointF origin, int width, int height, QString strategy)
                         qreal(width) / bounds.width(),
                         qreal(height) / bounds.height(),
                         actualStrategy, 0);
+    d->image->waitForDone();
 }
 
 void Node::rotateNode(double radians)
@@ -603,6 +638,7 @@ void Node::rotateNode(double radians)
     if (!d->node->parent()) return;
 
     d->image->rotateNode(d->node, radians, 0);
+    d->image->waitForDone();
 }
 
 void Node::cropNode(int x, int y, int w, int h)
@@ -613,6 +649,7 @@ void Node::cropNode(int x, int y, int w, int h)
 
     QRect rect = QRect(x, y, w, h);
     d->image->cropNode(d->node, rect);
+    d->image->waitForDone();
 }
 
 void Node::shearNode(double angleX, double angleY)
@@ -622,6 +659,7 @@ void Node::shearNode(double angleX, double angleY)
     if (!d->node->parent()) return;
 
     d->image->shearNode(d->node, angleX, angleY, 0);
+    d->image->waitForDone();
 }
 
 QImage Node::thumbnail(int w, int h)
