@@ -1,6 +1,6 @@
 /*  This file is part of the KDE project
     Copyright (c) 2005 Boudewijn Rempt <boud@valdyas.org>
-    Copyright (c) 2016 L. E. Segovia <leo.segovia@siggraph.org>
+    Copyright (c) 2016 L. E. Segovia <amy@amyspark.me>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -159,7 +159,7 @@ public /* methods */:
      * @brief getGroupNames
      * @return returns a list of group names, excluding the unsorted group.
      */
-    QStringList getGroupNames();
+    QStringList getGroupNames() const;
 
     /**
      * @brief getGroup
