@@ -119,6 +119,7 @@
 #include "kis_guides_config.h"
 #include "kis_image_barrier_lock_adapter.h"
 #include "KisReferenceImagesLayer.h"
+#include "dialogs/KisRecoverNamedAutosaveDialog.h"
 
 #include <mutex>
 #include "kis_config_notifier.h"
@@ -126,7 +127,8 @@
 #include "KisCloneDocumentStroke.h"
 
 #include <KisMirrorAxisConfig.h>
-
+#include <KisDecorationsWrapperLayer.h>
+#include "kis_simple_stroke_strategy.h"
 
 // Define the protocol used here for embedded documents' URL
 // This used to "store" but QUrl didn't like it,
@@ -230,11 +232,12 @@ private:
 class Q_DECL_HIDDEN KisDocument::Private
 {
 public:
-    Private(KisDocument *q)
-        : docInfo(new KoDocumentInfo(q)) // deleted by QObject
-        , importExportManager(new KisImportExportManager(q)) // deleted manually
-        , autoSaveTimer(new QTimer(q))
-        , undoStack(new UndoStack(q)) // deleted by QObject
+    Private(KisDocument *_q)
+        : q(_q)
+        , docInfo(new KoDocumentInfo(_q)) // deleted by QObject
+        , importExportManager(new KisImportExportManager(_q)) // deleted manually
+        , autoSaveTimer(new QTimer(_q))
+        , undoStack(new UndoStack(_q)) // deleted by QObject
         , m_bAutoDetectedMime(false)
         , modified(false)
         , readwrite(true)
@@ -251,36 +254,22 @@ public:
         } else {
             unit = KoUnit::Centimeter;
         }
+        connect(&imageIdleWatcher, SIGNAL(startedIdleMode()), q, SLOT(slotPerformIdleRoutines()));
     }
 
-    Private(const Private &rhs, KisDocument *q)
-        : docInfo(new KoDocumentInfo(*rhs.docInfo, q))
-        , unit(rhs.unit)
-        , importExportManager(new KisImportExportManager(q))
-        , mimeType(rhs.mimeType)
-        , outputMimeType(rhs.outputMimeType)
-        , autoSaveTimer(new QTimer(q))
-        , undoStack(new UndoStack(q))
-        , guidesConfig(rhs.guidesConfig)
-        , mirrorAxisConfig(rhs.mirrorAxisConfig)
-        , m_bAutoDetectedMime(rhs.m_bAutoDetectedMime)
-        , m_url(rhs.m_url)
-        , m_file(rhs.m_file)
-        , modified(rhs.modified)
-        , readwrite(rhs.readwrite)
-        , firstMod(rhs.firstMod)
-        , lastMod(rhs.lastMod)
+    Private(const Private &rhs, KisDocument *_q)
+        : q(_q)
+        , docInfo(new KoDocumentInfo(*rhs.docInfo, _q))
+        , importExportManager(new KisImportExportManager(_q))
+        , autoSaveTimer(new QTimer(_q))
+        , undoStack(new UndoStack(_q))
         , nserver(new KisNameServer(*rhs.nserver))
         , preActivatedNode(0) // the node is from another hierarchy!
         , imageIdleWatcher(2000 /*ms*/)
-        , assistants(rhs.assistants) // WARNING: assistants should not store pointers to the document!
-        , globalAssistantsColor(rhs.globalAssistantsColor)
-        , paletteList(rhs.paletteList)
-        , gridConfig(rhs.gridConfig)
         , savingLock(&savingMutex)
-        , batchMode(rhs.batchMode)
     {
-        // TODO: clone assistants
+        copyFromImpl(rhs, _q, CONSTRUCT);
+        connect(&imageIdleWatcher, SIGNAL(startedIdleMode()), q, SLOT(slotPerformIdleRoutines()));
     }
 
     ~Private() {
@@ -288,6 +277,7 @@ public:
         delete nserver;
     }
 
+    KisDocument *q = 0;
     KoDocumentInfo *docInfo = 0;
 
     KoUnit unit;
@@ -338,9 +328,8 @@ public:
 
     QColor globalAssistantsColor;
 
-    KisSharedPtr<KisReferenceImagesLayer> referenceImagesLayer;
-
     QList<KoColorSet*> paletteList;
+    bool ownsPaletteList = false;
 
     KisGridConfig gridConfig;
 
@@ -356,22 +345,126 @@ public:
 
     bool batchMode { false };
 
+    void syncDecorationsWrapperLayerState();
+
     void setImageAndInitIdleWatcher(KisImageSP _image) {
         image = _image;
 
         imageIdleWatcher.setTrackedImage(image);
-
-        if (image) {
-            imageIdleConnection.reset(
-                        new KisSignalAutoConnection(
-                            &imageIdleWatcher, SIGNAL(startedIdleMode()),
-                            image.data(), SLOT(explicitRegenerateLevelOfDetail())));
-        }
     }
 
+    void copyFrom(const Private &rhs, KisDocument *q);
+    void copyFromImpl(const Private &rhs, KisDocument *q, KisDocument::CopyPolicy policy);
+
+    /// clones the palette list oldList
+    /// the ownership of the returned KoColorSet * belongs to the caller
+    QList<KoColorSet *> clonePaletteList(const QList<KoColorSet *> &oldList);
+
     class StrippedSafeSavingLocker;
 };
 
+
+void KisDocument::Private::syncDecorationsWrapperLayerState()
+{
+    if (!this->image) return;
+
+    KisImageSP image = this->image;
+    KisDecorationsWrapperLayerSP decorationsLayer =
+        KisLayerUtils::findNodeByType<KisDecorationsWrapperLayer>(image->root());
+
+    const bool needsDecorationsWrapper =
+            gridConfig.showGrid() || (guidesConfig.showGuides() && guidesConfig.hasGuides()) || !assistants.isEmpty();
+
+    struct SyncDecorationsWrapperStroke : public KisSimpleStrokeStrategy {
+        SyncDecorationsWrapperStroke(KisDocument *document, bool needsDecorationsWrapper)
+            : KisSimpleStrokeStrategy(QLatin1String("sync-decorations-wrapper"),
+                                      kundo2_noi18n("start-isolated-mode")),
+              m_document(document),
+              m_needsDecorationsWrapper(needsDecorationsWrapper)
+        {
+            this->enableJob(JOB_INIT, true, KisStrokeJobData::SEQUENTIAL, KisStrokeJobData::EXCLUSIVE);
+            setClearsRedoOnStart(false);
+        }
+
+        void initStrokeCallback() {
+            KisDecorationsWrapperLayerSP decorationsLayer =
+                KisLayerUtils::findNodeByType<KisDecorationsWrapperLayer>(m_document->image()->root());
+
+            if (m_needsDecorationsWrapper && !decorationsLayer) {
+                m_document->image()->addNode(new KisDecorationsWrapperLayer(m_document));
+            } else if (!m_needsDecorationsWrapper && decorationsLayer) {
+                m_document->image()->removeNode(decorationsLayer);
+            }
+        }
+
+    private:
+        KisDocument *m_document = 0;
+        bool m_needsDecorationsWrapper = false;
+    };
+
+    KisStrokeId id = image->startStroke(new SyncDecorationsWrapperStroke(q, needsDecorationsWrapper));
+    image->endStroke(id);
+}
+
+void KisDocument::Private::copyFrom(const Private &rhs, KisDocument *q)
+{
+    copyFromImpl(rhs, q, KisDocument::REPLACE);
+}
+
+void KisDocument::Private::copyFromImpl(const Private &rhs, KisDocument *q, KisDocument::CopyPolicy policy)
+{
+    if (policy == REPLACE) {
+        delete docInfo;
+    }
+    docInfo = (new KoDocumentInfo(*rhs.docInfo, q));
+    unit = rhs.unit;
+    mimeType = rhs.mimeType;
+    outputMimeType = rhs.outputMimeType;
+
+    if (policy == REPLACE) {
+        q->setGuidesConfig(rhs.guidesConfig);
+        q->setMirrorAxisConfig(rhs.mirrorAxisConfig);
+        q->setModified(rhs.modified);
+        q->setAssistants(KisPaintingAssistant::cloneAssistantList(rhs.assistants));
+        q->setGridConfig(rhs.gridConfig);
+    } else {
+        // in CONSTRUCT mode, we cannot use the functions of KisDocument
+        // because KisDocument does not yet have a pointer to us.
+        guidesConfig = rhs.guidesConfig;
+        mirrorAxisConfig = rhs.mirrorAxisConfig;
+        modified = rhs.modified;
+        assistants = KisPaintingAssistant::cloneAssistantList(rhs.assistants);
+        gridConfig = rhs.gridConfig;
+    }
+    m_bAutoDetectedMime = rhs.m_bAutoDetectedMime;
+    m_url = rhs.m_url;
+    m_file = rhs.m_file;
+    readwrite = rhs.readwrite;
+    firstMod = rhs.firstMod;
+    lastMod = rhs.lastMod;
+    // XXX: the display properties will be shared between different snapshots
+    globalAssistantsColor = rhs.globalAssistantsColor;
+
+    if (policy == REPLACE) {
+        QList<KoColorSet *> newPaletteList = clonePaletteList(rhs.paletteList);
+        q->setPaletteList(newPaletteList, /* emitSignal = */ true);
+        // we still do not own palettes if we did not
+    } else {
+        paletteList = rhs.paletteList;
+    }
+
+    batchMode = rhs.batchMode;
+}
+
+QList<KoColorSet *> KisDocument::Private::clonePaletteList(const QList<KoColorSet *> &oldList)
+{
+    QList<KoColorSet *> newList;
+    Q_FOREACH (KoColorSet *palette, oldList) {
+        newList << new KoColorSet(*palette);
+    }
+    return newList;
+}
+
 class KisDocument::Private::StrippedSafeSavingLocker {
 public:
     StrippedSafeSavingLocker(QMutex *savingMutex, KisImageSP image)
@@ -441,27 +534,7 @@ KisDocument::KisDocument(const KisDocument &rhs)
     : QObject(),
       d(new Private(*rhs.d, this))
 {
-    connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(slotConfigChanged()));
-    connect(d->undoStack, SIGNAL(cleanChanged(bool)), this, SLOT(slotUndoStackCleanChanged(bool)));
-    connect(d->autoSaveTimer, SIGNAL(timeout()), this, SLOT(slotAutoSave()));
-    setObjectName(rhs.objectName());
-
-    d->shapeController = new KisShapeController(this, d->nserver);
-    d->koShapeController = new KoShapeController(0, d->shapeController);
-    d->shapeController->resourceManager()->setGlobalShapeController(d->koShapeController);
-
-    slotConfigChanged();
-
-    // clone the image with keeping the GUIDs of the layers intact
-    // NOTE: we expect the image to be locked!
-    setCurrentImage(rhs.image()->clone(true), false);
-
-    if (rhs.d->preActivatedNode) {
-        // since we clone uuid's, we can use them for lacating new
-        // nodes. Otherwise we would need to use findSymmetricClone()
-        d->preActivatedNode =
-                KisLayerUtils::findNodeByUuid(d->image->root(), rhs.d->preActivatedNode->uuid());
-    }
+    copyFromDocumentImpl(rhs, CONSTRUCT);
 }
 
 KisDocument::~KisDocument()
@@ -517,6 +590,10 @@ KisDocument::~KisDocument()
         KIS_SAFE_ASSERT_RECOVER_NOOP(!sanityCheckPointer.isValid());
     }
 
+    if (d->ownsPaletteList) {
+        qDeleteAll(d->paletteList);
+    }
+
     delete d;
 }
 
@@ -555,6 +632,12 @@ bool KisDocument::exportDocumentImpl(const KritaUtils::ExportFileJob &job, KisPr
             backupDir = QStandardPaths::writableLocation(QStandardPaths::TempLocation);
             break;
         default:
+#ifdef Q_OS_ANDROID
+            // We deal with URIs, there may or may not be a "directory"
+            backupDir = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation).append("/krita-backup");
+            QDir().mkpath(backupDir);
+#endif
+
             // Do nothing: the empty string is user file location
             break;
         }
@@ -594,14 +677,14 @@ bool KisDocument::exportDocumentImpl(const KritaUtils::ExportFileJob &job, KisPr
     }
 
     const QString actionName =
-            job.flags & KritaUtils::SaveIsExporting ?
-                i18n("Exporting Document...") :
-                i18n("Saving Document...");
+        job.flags & KritaUtils::SaveIsExporting ?
+        i18n("Exporting Document...") :
+        i18n("Saving Document...");
 
     bool started =
-            initiateSavingInBackground(actionName,
-                                       this, SLOT(slotCompleteSavingDocument(KritaUtils::ExportFileJob, KisImportExportErrorCode ,QString)),
-                                       job, exportConfiguration);
+        initiateSavingInBackground(actionName,
+                                   this, SLOT(slotCompleteSavingDocument(KritaUtils::ExportFileJob, KisImportExportErrorCode ,QString)),
+                                   job, exportConfiguration);
     if (!started) {
         emit canceled(QString());
     }
@@ -628,11 +711,11 @@ bool KisDocument::exportDocument(const QUrl &url, const QByteArray &mimeType, bo
                         .arg(d->image->animationInterface()->framerate())
                         .arg(exportConfiguration ? exportConfiguration->toXML() : "No configuration"));
 
-    return exportDocumentImpl(KritaUtils::ExportFileJob(url.toLocalFile(),
+
+    return exportDocumentImpl(KritaUtils::ExportFileJob(toPath(url),
                                                         mimeType,
                                                         flags),
                               exportConfiguration);
-
 }
 
 bool KisDocument::saveAs(const QUrl &_url, const QByteArray &mimeType, bool showWarnings, KisPropertiesConfigurationSP exportConfiguration)
@@ -651,7 +734,7 @@ bool KisDocument::saveAs(const QUrl &_url, const QByteArray &mimeType, bool show
                         .arg(url().toLocalFile()));
 
 
-    return exportDocumentImpl(ExportFileJob(_url.toLocalFile(),
+    return exportDocumentImpl(ExportFileJob(toPath(_url),
                                             mimeType,
                                             showWarnings ? SaveShowWarnings : SaveNone),
                               exportConfiguration);
@@ -779,13 +862,96 @@ KisDocument* KisDocument::lockAndCloneForSaving()
     return new KisDocument(*this);
 }
 
+KisDocument *KisDocument::lockAndCreateSnapshot()
+{
+    KisDocument *doc = lockAndCloneForSaving();
+    if (doc) {
+        // clone palette list
+        doc->d->paletteList = doc->d->clonePaletteList(doc->d->paletteList);
+        doc->d->ownsPaletteList = true;
+    }
+    return doc;
+}
+
+void KisDocument::copyFromDocument(const KisDocument &rhs)
+{
+    copyFromDocumentImpl(rhs, REPLACE);
+}
+
+void KisDocument::copyFromDocumentImpl(const KisDocument &rhs, CopyPolicy policy)
+{
+    if (policy == REPLACE) {
+        d->copyFrom(*(rhs.d), this);
+
+        d->undoStack->clear();
+    } else {
+        // in CONSTRUCT mode, d should be already initialized
+        connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(slotConfigChanged()));
+        connect(d->undoStack, SIGNAL(cleanChanged(bool)), this, SLOT(slotUndoStackCleanChanged(bool)));
+        connect(d->autoSaveTimer, SIGNAL(timeout()), this, SLOT(slotAutoSave()));
+
+        d->shapeController = new KisShapeController(this, d->nserver);
+        d->koShapeController = new KoShapeController(0, d->shapeController);
+        d->shapeController->resourceManager()->setGlobalShapeController(d->koShapeController);
+    }
+
+    setObjectName(rhs.objectName());
+
+    slotConfigChanged();
+
+    if (rhs.d->image) {
+        if (policy == REPLACE) {
+            d->image->barrierLock(/* readOnly = */ false);
+            rhs.d->image->barrierLock(/* readOnly = */ true);
+            d->image->copyFromImage(*(rhs.d->image));
+            d->image->unlock();
+            rhs.d->image->unlock();
+            setCurrentImage(d->image, /* forceInitialUpdate = */ true);
+        } else {
+            // clone the image with keeping the GUIDs of the layers intact
+            // NOTE: we expect the image to be locked!
+            setCurrentImage(rhs.image()->clone(/* exactCopy = */ true), /* forceInitialUpdate = */ false);
+        }
+    }
+
+    if (rhs.d->preActivatedNode) {
+        QQueue<KisNodeSP> linearizedNodes;
+        KisLayerUtils::recursiveApplyNodes(rhs.d->image->root(),
+                                           [&linearizedNodes](KisNodeSP node) {
+                                               linearizedNodes.enqueue(node);
+                                           });
+        KisLayerUtils::recursiveApplyNodes(d->image->root(),
+                                           [&linearizedNodes, &rhs, this](KisNodeSP node) {
+                                               KisNodeSP refNode = linearizedNodes.dequeue();
+                                               if (rhs.d->preActivatedNode.data() == refNode.data()) {
+                                                   d->preActivatedNode = node;
+                                               }
+                                           });
+    }
+
+    // reinitialize references' signal connection
+    KisReferenceImagesLayerSP referencesLayer = this->referenceImagesLayer();
+    setReferenceImagesLayer(referencesLayer, false);
+
+    KisDecorationsWrapperLayerSP decorationsLayer =
+        KisLayerUtils::findNodeByType<KisDecorationsWrapperLayer>(d->image->root());
+    if (decorationsLayer) {
+        decorationsLayer->setDocument(this);
+    }
+
+
+    if (policy == REPLACE) {
+        setModified(true);
+    }
+}
+
 bool KisDocument::exportDocumentSync(const QUrl &url, const QByteArray &mimeType, KisPropertiesConfigurationSP exportConfiguration)
 {
     {
 
         /**
-         * The caller guarantees that noone else uses the document (usually,
-         * it is a temporary docuent created specifically for exporting), so
+         * The caller guarantees that no one else uses the document (usually,
+         * it is a temporary document created specifically for exporting), so
          * we don't need to copy or lock the document. Instead we should just
          * ensure the barrier lock is synced and then released.
          */
@@ -860,6 +1026,13 @@ bool KisDocument::initiateSavingInBackground(const QString actionName,
         waitForImage(clonedDocument->image());
     }
 
+    KisConfig cfg(true);
+    if (cfg.trimKra()) {
+        clonedDocument->image()->cropImage(clonedDocument->image()->bounds());
+        clonedDocument->image()->purgeUnusedData(false);
+        waitForImage(clonedDocument->image());
+    }
+
     KIS_SAFE_ASSERT_RECOVER(clonedDocument->image()->isIdle()) {
         waitForImage(clonedDocument->image());
     }
@@ -938,7 +1111,7 @@ void KisDocument::slotChildCompletedSavingInBackground(KisImportExportErrorCode
                         .arg(QString::fromLatin1(job.mimeType))
                         .arg(!status.isOk() ? exportErrorToUserMessage(status, errorMessage) : "OK")
                         .arg(fi.size())
-                        .arg(QString::fromLatin1(KoMD5Generator().generateHash(job.filePath).toHex())));
+                        .arg(fi.size() > 10000000 ? "FILE_BIGGER_10MB" : QString::fromLatin1(KoMD5Generator().generateHash(job.filePath).toHex())));
 
     emit sigCompleteBackgroundSaving(job, status, errorMessage);
 }
@@ -993,6 +1166,18 @@ void KisDocument::slotInitiateAsyncAutosaving(KisDocument *clonedDocument)
     slotAutoSaveImpl(std::unique_ptr<KisDocument>(clonedDocument));
 }
 
+void KisDocument::slotPerformIdleRoutines()
+{
+    d->image->explicitRegenerateLevelOfDetail();
+
+
+    /// TODO: automatical purging is disabled for now: it modifies
+    ///       data managers without creating a transaction, which breaks
+    ///       undo.
+
+    // d->image->purgeUnusedData(true);
+}
+
 void KisDocument::slotCompleteAutoSaving(const KritaUtils::ExportFileJob &job, KisImportExportErrorCode status, const QString &errorMessage)
 {
     Q_UNUSED(job);
@@ -1168,6 +1353,15 @@ QString KisDocument::generateAutoSaveFileName(const QString & path) const
 
     QFileInfo fi(path);
     QString dir = fi.absolutePath();
+
+#ifdef Q_OS_ANDROID
+    // URIs may or may not have a directory backing them, so we save to our default autosave location
+    if (path.startsWith("content://")) {
+        dir = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation).append("/krita-backup");
+        QDir().mkpath(dir);
+    }
+#endif
+
     QString filename = fi.fileName();
 
     if (path.isEmpty() || autosavePattern1.match(filename).hasMatch() || autosavePattern2.match(filename).hasMatch() || !fi.isWritable()) {
@@ -1210,9 +1404,11 @@ bool KisDocument::importDocument(const QUrl &_url)
 
 bool KisDocument::openUrl(const QUrl &_url, OpenFlags flags)
 {
+#ifndef Q_OS_ANDROID
     if (!_url.isLocalFile()) {
         return false;
     }
+#endif
     dbgUI << "url=" << _url.url();
     d->lastErrorMessage.clear();
 
@@ -1223,6 +1419,7 @@ bool KisDocument::openUrl(const QUrl &_url, OpenFlags flags)
     }
 
     QUrl url(_url);
+    QString original  = "";
     bool autosaveOpened = false;
     if (url.isLocalFile() && !fileBatchMode()) {
         QString file = url.toLocalFile();
@@ -1232,16 +1429,17 @@ bool KisDocument::openUrl(const QUrl &_url, OpenFlags flags)
             kisApp->hideSplashScreen();
             //qDebug() <<"asf=" << asf;
             // ## TODO compare timestamps ?
-            int res = QMessageBox::warning(0,
-                                           i18nc("@title:window", "Krita"),
-                                           i18n("An autosaved file exists for this document.\nDo you want to open the autosaved file instead?"),
-                                           QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel, QMessageBox::Yes);
+            KisRecoverNamedAutosaveDialog dlg(0, file, asf);
+            dlg.exec();
+            int res = dlg.result();
+
             switch (res) {
-            case QMessageBox::Yes :
+            case KisRecoverNamedAutosaveDialog::OpenAutosave :
+                original = file;
                 url.setPath(asf);
                 autosaveOpened = true;
                 break;
-            case QMessageBox::No :
+            case KisRecoverNamedAutosaveDialog::OpenMainFile :
                 KisUsageLogger::log(QString("Removing autosave file: %1").arg(asf));
                 QFile::remove(asf);
                 break;
@@ -1257,6 +1455,9 @@ bool KisDocument::openUrl(const QUrl &_url, OpenFlags flags)
         setReadWrite(true); // enable save button
         setModified(true);
         setRecovered(true);
+
+        setUrl(QUrl::fromLocalFile(original)); // since it was an autosave, it will be a local file
+        setLocalFilePath(original);
     }
     else {
         if (ret) {
@@ -1265,8 +1466,7 @@ bool KisDocument::openUrl(const QUrl &_url, OpenFlags flags)
                 KisPart::instance()->addRecentURLToAllMainWindows(_url);
             }
 
-            // Detect readonly local-files; remote files are assumed to be writable
-            QFileInfo fi(url.toLocalFile());
+            QFileInfo fi(toPath(url));
             setReadWrite(fi.isWritable());
         }
 
@@ -1316,10 +1516,12 @@ public:
 bool KisDocument::openFile()
 {
     //dbgUI <<"for" << localFilePath();
+#ifndef Q_OS_ANDROID
     if (!QFile::exists(localFilePath()) && !fileBatchMode()) {
         QMessageBox::critical(0, i18nc("@title:window", "Krita"), i18n("File %1 does not exist.", localFilePath()));
         return false;
     }
+#endif
 
     QString filename = localFilePath();
     QString typeName = mimeType();
@@ -1328,8 +1530,6 @@ bool KisDocument::openFile()
         typeName = KisMimeDatabase::mimeTypeForFile(filename);
     }
 
-    //qDebug() << "mimetypes 4:" << typeName;
-
     // Allow to open backup files, don't keep the mimetype application/x-trash.
     if (typeName == "application/x-trash") {
         QString path = filename;
@@ -1376,6 +1576,7 @@ bool KisDocument::openFile()
     }
 
     setMimeTypeAfterLoading(typeName);
+    d->syncDecorationsWrapperLayerState();
     emit sigLoadingFinished();
 
     undoStack()->clear();
@@ -1383,6 +1584,36 @@ bool KisDocument::openFile()
     return true;
 }
 
+void KisDocument::autoSaveOnPause()
+{
+    if (!d->modified || !d->modifiedAfterAutosave)
+        return;
+
+    const QString autoSaveFileName = generateAutoSaveFileName(localFilePath());
+
+    QUrl url("file:/" + autoSaveFileName);
+    bool started = exportDocumentSync(url, nativeFormatMimeType());
+
+    if (started)
+    {
+        d->modifiedAfterAutosave = false;
+        dbgAndroid << "autoSaveOnPause successful";
+    }
+    else
+    {
+        qWarning() << "Could not auto-save when paused";
+    }
+}
+
+QString KisDocument::toPath(const QUrl &url) const
+{
+#ifdef Q_OS_ANDROID
+    return (url.toLocalFile().isEmpty()) ? url.toString() : url.toLocalFile();
+#else
+    return url.toLocalFile();
+#endif
+}
+
 // shared between openFile and koMainWindow's "create new empty document" code
 void KisDocument::setMimeTypeAfterLoading(const QString& mimeType)
 {
@@ -1632,6 +1863,11 @@ void KisDocument::slotConfigChanged()
     setNormalAutoSaveInterval();
 }
 
+void KisDocument::slotImageRootChanged()
+{
+    d->syncDecorationsWrapperLayerState();
+}
+
 void KisDocument::clearUndoHistory()
 {
     d->undoStack->clear();
@@ -1644,7 +1880,11 @@ KisGridConfig KisDocument::gridConfig() const
 
 void KisDocument::setGridConfig(const KisGridConfig &config)
 {
-    d->gridConfig = config;
+    if (d->gridConfig != config) {
+        d->gridConfig = config;
+        d->syncDecorationsWrapperLayerState();
+        emit sigGridConfigChanged(config);
+    }
 }
 
 QList<KoColorSet *> &KisDocument::paletteList()
@@ -1652,9 +1892,15 @@ QList<KoColorSet *> &KisDocument::paletteList()
     return d->paletteList;
 }
 
-void KisDocument::setPaletteList(const QList<KoColorSet *> &paletteList)
+void KisDocument::setPaletteList(const QList<KoColorSet *> &paletteList, bool emitSignal)
 {
-    d->paletteList = paletteList;
+    if (d->paletteList != paletteList) {
+        QList<KoColorSet *> oldPaletteList = d->paletteList;
+        d->paletteList = paletteList;
+        if (emitSignal) {
+            emit sigPaletteListChanged(oldPaletteList, paletteList);
+        }
+    }
 }
 
 const KisGuidesConfig& KisDocument::guidesConfig() const
@@ -1667,6 +1913,7 @@ void KisDocument::setGuidesConfig(const KisGuidesConfig &data)
     if (d->guidesConfig == data) return;
 
     d->guidesConfig = data;
+    d->syncDecorationsWrapperLayerState();
     emit sigGuidesConfigChanged(d->guidesConfig);
 }
 
@@ -1769,8 +2016,13 @@ bool KisDocument::openUrlInternal(const QUrl &url)
 
     d->m_file.clear();
 
+#ifndef Q_OS_ANDROID
     if (d->m_url.isLocalFile()) {
         d->m_file = d->m_url.toLocalFile();
+#else
+        d->m_file = toPath(d->m_url);
+#endif
+
         bool ret;
         // set the mimetype only if it was not already set (for example, by the host application)
         if (d->mimeType.isEmpty()) {
@@ -1790,8 +2042,10 @@ bool KisDocument::openUrlInternal(const QUrl &url)
             emit canceled(QString());
         }
         return ret;
+#ifndef Q_OS_ANDROID
     }
     return false;
+#endif
 }
 
 bool KisDocument::newImage(const QString& name,
@@ -1916,34 +2170,53 @@ QList<KisPaintingAssistantSP> KisDocument::assistants() const
 
 void KisDocument::setAssistants(const QList<KisPaintingAssistantSP> &value)
 {
-    d->assistants = value;
+    if (d->assistants != value) {
+        d->assistants = value;
+        d->syncDecorationsWrapperLayerState();
+        emit sigAssistantsChanged();
+    }
 }
 
-KisSharedPtr<KisReferenceImagesLayer> KisDocument::referenceImagesLayer() const
+KisReferenceImagesLayerSP KisDocument::referenceImagesLayer() const
 {
-    return d->referenceImagesLayer.data();
+    if (!d->image) return KisReferenceImagesLayerSP();
+
+    KisReferenceImagesLayerSP referencesLayer =
+        KisLayerUtils::findNodeByType<KisReferenceImagesLayer>(d->image->root());
+
+    return referencesLayer;
 }
 
 void KisDocument::setReferenceImagesLayer(KisSharedPtr<KisReferenceImagesLayer> layer, bool updateImage)
 {
-    if (d->referenceImagesLayer) {
-        d->referenceImagesLayer->disconnect(this);
+    KisReferenceImagesLayerSP currentReferenceLayer = referenceImagesLayer();
+
+    if (currentReferenceLayer == layer) {
+        return;
+    }
+
+    if (currentReferenceLayer) {
+        currentReferenceLayer->disconnect(this);
     }
 
     if (updateImage) {
+        if (currentReferenceLayer) {
+            d->image->removeNode(currentReferenceLayer);
+        }
+
         if (layer) {
             d->image->addNode(layer);
-        } else {
-            d->image->removeNode(d->referenceImagesLayer);
         }
     }
 
-    d->referenceImagesLayer = layer;
+    currentReferenceLayer = layer;
 
-    if (d->referenceImagesLayer) {
-        connect(d->referenceImagesLayer, SIGNAL(sigUpdateCanvas(QRectF)),
+    if (currentReferenceLayer) {
+        connect(currentReferenceLayer, SIGNAL(sigUpdateCanvas(QRectF)),
                 this, SIGNAL(sigReferenceImagesChanged()));
     }
+
+    emit sigReferenceImagesLayerChanged(layer);
 }
 
 void KisDocument::setPreActivatedNode(KisNodeSP activatedNode)
@@ -1984,6 +2257,7 @@ void KisDocument::setCurrentImage(KisImageSP image, bool forceInitialUpdate)
     d->shapeController->setImage(image);
     setModified(false);
     connect(d->image, SIGNAL(sigImageModified()), this, SLOT(setImageModified()), Qt::UniqueConnection);
+    connect(d->image, SIGNAL(sigLayersChangedAsync()), this, SLOT(slotImageRootChanged()));
 
     if (forceInitialUpdate) {
         d->image->initialRefreshGraph();
@@ -2036,8 +2310,10 @@ QRectF KisDocument::documentBounds() const
 {
     QRectF bounds = d->image->bounds();
 
-    if (d->referenceImagesLayer) {
-        bounds |= d->referenceImagesLayer->boundingImageRect();
+    KisReferenceImagesLayerSP referenceImagesLayer = this->referenceImagesLayer();
+
+    if (referenceImagesLayer) {
+        bounds |= referenceImagesLayer->boundingImageRect();
     }
 
     return bounds;
