@@ -90,7 +90,7 @@ public:
     /**
      * Add the specified document to the list of documents this KisPart manages.
      */
-    void addDocument(KisDocument *document);
+    void addDocument(KisDocument *document, bool notify = true);
 
     /**
      * @return a list of all documents this part manages
@@ -102,7 +102,7 @@ public:
      */
     int documentCount() const;
 
-    void removeDocument(KisDocument *document);
+    void removeDocument(KisDocument *document, bool deleteDocument = true);
 
     // ----------------- MainWindow management -----------------
 
@@ -112,6 +112,13 @@ public:
      */
     KisMainWindow *createMainWindow(QUuid id = QUuid());
 
+    /**
+     * @brief notifyMainWindowIsBeingCreated emits the sigMainWindowCreated signal
+     * @param mainWindow
+     */
+    void notifyMainWindowIsBeingCreated(KisMainWindow *mainWindow);
+
+
     /**
      * Removes a main window from the list of managed windows.
      *
@@ -143,11 +150,20 @@ public:
      */
     KisIdleWatcher *idleWatcher() const;
 
+    // ----------------- Cache Populator Management -----------------
     /**
      * @return the application-wide AnimationCachePopulator.
      */
     KisAnimationCachePopulator *cachePopulator() const;
 
+    /**
+     * Adds a frame time index to a priority stack, which should be
+     * cached immediately and irregardless of whether it is the
+     * the currently occupied frame. The process of regeneration is
+     * started immediately.
+     */
+    void prioritizeFrameForCache(KisImageSP image, int frame);
+
 public Q_SLOTS:
 
     /**
@@ -192,7 +208,7 @@ Q_SIGNALS:
     void sigDocumentAdded(KisDocument *document);
     void sigDocumentSaved(const QString &url);
     void sigDocumentRemoved(const QString &filename);
-    void sigWindowAdded(KisMainWindow *window);
+    void sigMainWindowIsBeingCreated(KisMainWindow *window);
 
 public:
 
@@ -205,8 +221,7 @@ public:
      * views, and if the document wasn't known yet, it's registered as well.
      */
     KisView *createView(KisDocument *document,
-                        KoCanvasResourceProvider *resourceManager,
-                        KActionCollection *actionCollection,
+                        KisViewManager *viewManager,
                         QWidget *parent);
 
     /**
@@ -258,6 +273,14 @@ public:
      */
     bool closingSession() const;
 
+    /**
+     * This function returns true if the instance has already been initialized,
+     * false otherwise. This to prevent premature initialization that causes crash
+     * on android see `1fbb25506a`
+     * @see QGlobalStatic::exists()
+     */
+    static bool exists();
+
 private Q_SLOTS:
 
     void slotDocumentSaved();
