@@ -24,7 +24,6 @@
 #include <QMimeData>
 #include <QPixmap>
 #include <QImage>
-#include <QThread>
 
 #include <KisMimeDatabase.h>
 #include "kis_action_manager.h"
@@ -36,6 +35,7 @@
 
 #include <QListWidget>
 #include <QListWidgetItem>
+
 #include "kis_icon_utils.h"
 #include "krita_utils.h"
 #include "KoStore.h"
@@ -44,6 +44,7 @@
 #include <kis_image.h>
 #include <kis_paint_device.h>
 #include <KisPart.h>
+#include <utils/KisFileIconCreator.h>
 
 #include <kritaversion.h>
 #include <QSysInfo>
@@ -51,55 +52,19 @@
 #include <kis_image_config.h>
 #include "opengl/kis_opengl.h"
 
+#ifdef Q_OS_ANDROID
+#include <QtAndroid>
 
 
+QPushButton* KisWelcomePageWidget::donationLink;
+QLabel* KisWelcomePageWidget::donationBannerImage;
+#endif
+
 
 KisWelcomePageWidget::KisWelcomePageWidget(QWidget *parent)
     : QWidget(parent)
 {
     setupUi(this);
-
-#ifdef FALSE
-    lblSurveyLink->setVisible(true);
-    lblSurveyLink->setTextInteractionFlags(Qt::TextBrowserInteraction);
-
-    QString surveyId = "786736";
-    QString arguments;
-    arguments += "&KritaVersion=" + QStringLiteral(KRITA_VERSION_STRING);
-    arguments += "&BuildAbi=" + QSysInfo::buildAbi();
-    arguments += "&BuildCPUArchitecture=" + QSysInfo::buildCpuArchitecture();
-    arguments += "&KernelVersion=" + QSysInfo::kernelVersion();
-    arguments += "&ProductType=" + QSysInfo::productType();
-    arguments += "&ProductVersion=" + QSysInfo::productVersion();
-    arguments += "&InstantPreviewEnable=" + QString(KisConfig(true).levelOfDetailEnabled( )? "true" : "false");
-    arguments += "&TotalMemorySize=" + QString::number(KisImageConfig(true).totalRAM());
-    arguments += "&SwapLocation=" + KisImageConfig(true).swapDir();
-    arguments += "&NumberOfCores=" + QString::number(QThread::idealThreadCount());
-    arguments += "&HiDPI=" + QString(QCoreApplication::testAttribute(Qt::AA_EnableHighDpiScaling) ? "true" : "false");
-    arguments += "&OpenGLRenderer=" + KisOpenGL::convertOpenGLRendererToConfig(KisOpenGL::getCurrentOpenGLRenderer());
-
-    QString openGLInfo = KisOpenGL::getDebugText();
-    QStringList infolines = openGLInfo.split("\n");
-    Q_FOREACH(const QString line, infolines) {
-        if (line.contains("Vendor")) {
-            arguments += "&GPUVendor=" + line.split(": ")[1].replace("\n", "");
-        }
-        if (line.contains("Version")) {
-            arguments += "&OpenGLVersion=" + line.split(": ")[1].replace("\n", "");
-        }
-    }
-    QUrl url (arguments);
-
-    QString source = QString("<html><head/><body><p align=\"center\"><a href=\"https://survey.kde.org/index.php/%1?lang=en?%2\">"
-                             "<span style=\"font-size:18pt; font-weight:600; text-decoration: underline; color:#2980b9;\">"
-                             "Help Krita, do the survey!"
-                              "</span></a></p></body></html>").arg(surveyId).arg(QString::fromUtf8(url.toEncoded()));
-
-    lblSurveyLink->setText(source);
-#else
-    lblSurveyLink->setVisible(false);
-#endif
-
     recentDocumentsListView->setDragEnabled(false);
     recentDocumentsListView->viewport()->setAutoFillBackground(false);
     recentDocumentsListView->setSpacing(2);
@@ -138,8 +103,53 @@ KisWelcomePageWidget::KisWelcomePageWidget(QWidget *parent)
     kdeIcon->setIcon(KisIconUtils::loadIcon(QStringLiteral("kde")).pixmap(20));
 
 
+    versionNotificationLabel->setTextFormat(Qt::RichText);
+    versionNotificationLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
+    versionNotificationLabel->setOpenExternalLinks(true);
+
     connect(chkShowNews, SIGNAL(toggled(bool)), newsWidget, SLOT(toggleNews(bool)));
 
+    connect(newsWidget, SIGNAL(newsDataChanged()), this, SLOT(slotUpdateVersionMessage()));
+
+#ifdef Q_OS_ANDROID
+    // enabling this widgets crashes the app, so it is better for it to be hidden for now
+    newsWidget->hide();
+    helpTitleLabel_2->hide();
+    chkShowNews->hide();
+
+    donationLink = new QPushButton(dropFrameBorder);
+    donationLink->setFlat(true);
+    QFont f = font();
+    f.setPointSize(15);
+    f.setUnderline(true);
+    donationLink->setFont(f);
+
+    connect(donationLink, SIGNAL(clicked(bool)), this, SLOT(slotStartDonationFlow()));
+
+    verticalLayout_3->addWidget(donationLink);
+    verticalLayout_3->setAlignment(donationLink, Qt::AlignTop);
+    verticalLayout_3->setSpacing(20);
+
+    donationBannerImage = new QLabel(dropFrameBorder);
+    QString bannerPath = QStandardPaths::locate(QStandardPaths::AppDataLocation, "share/krita/donation/banner.png");
+    donationBannerImage->setPixmap(QPixmap(bannerPath));
+
+    verticalLayout_3->addWidget(donationBannerImage);
+
+    jboolean bannerPurchased = QAndroidJniObject::callStaticMethod<jboolean>("org/krita/android/DonationHelper", "isBadgePurchased", "()Z");
+    if (bannerPurchased) {
+        donationLink->hide();
+        donationBannerImage->show();
+        QAndroidJniObject::callStaticMethod<void>("org/krita/android/DonationHelper", "endConnection", "()V");
+    } else {
+        donationLink->show();
+        donationBannerImage->hide();
+    }
+
+#endif
+
+
+
     // configure the News area
     KisConfig cfg(true);
     bool m_getNews = cfg.readEntry<bool>("FetchNews", false);
@@ -180,6 +190,7 @@ void KisWelcomePageWidget::setMainWindow(KisMainWindow* mainWin)
     }
 }
 
+
 void KisWelcomePageWidget::showDropAreaIndicator(bool show)
 {
     if (!show) {
@@ -199,12 +210,12 @@ void KisWelcomePageWidget::showDropAreaIndicator(bool show)
 void KisWelcomePageWidget::slotUpdateThemeColors()
 {
 
-    QColor textColor = qApp->palette().color(QPalette::Text);
-    QColor backgroundColor = qApp->palette().color(QPalette::Background);
+    textColor = qApp->palette().color(QPalette::Text);
+    backgroundColor = qApp->palette().color(QPalette::Background);
 
     // make the welcome screen labels a subtle color so it doesn't clash with the main UI elements
-    QColor blendedColor = KritaUtils::blendColors(textColor, backgroundColor, 0.8);
-    QString blendedStyle = QString("color: ").append(blendedColor.name());
+    blendedColor = KritaUtils::blendColors(textColor, backgroundColor, 0.8);
+    blendedStyle = QString("color: ").append(blendedColor.name());
 
 
     // what labels to change the color...
@@ -263,6 +274,15 @@ void KisWelcomePageWidget::slotUpdateThemeColors()
 
     poweredByKDELink->setText(QString("<a style=\"color: " + blendedColor.name() + " \" href=\"https://userbase.kde.org/What_is_KDE?" + analyticsString + "what-is-kde" + "\">")
                               .append(i18n("Powered by KDE")).append("</a>"));
+
+
+    slotUpdateVersionMessage(); // text set from RSS feed
+
+
+#ifdef Q_OS_ANDROID
+    donationLink->setStyleSheet(blendedStyle);
+    donationLink->setText(QString(i18n("Get your Krita Supporter Badge here!")));
+#endif
     // re-populate recent files since they might have themed icons
     populateRecentDocuments();
 
@@ -275,6 +295,7 @@ void KisWelcomePageWidget::populateRecentDocuments()
 
     // grab recent files data
     int numRecentFiles = m_mainWindow->recentFilesUrls().length() > 5 ? 5 : m_mainWindow->recentFilesUrls().length(); // grab at most 5
+    KisFileIconCreator iconCreator;
 
     for (int i = 0; i < numRecentFiles; i++ ) {
 
@@ -287,69 +308,19 @@ void KisWelcomePageWidget::populateRecentDocuments()
 
         QList<QUrl> brokenUrls;
 
+
         if (m_thumbnailMap.contains(recentFileUrlPath)) {
             recentItem->setIcon(m_thumbnailMap[recentFileUrlPath]);
-        }
-        else {
-            QFileInfo fi(recentFileUrlPath);
-            if (fi.exists()) {
-                QString mimeType = KisMimeDatabase::mimeTypeForFile(recentFileUrlPath);
-                if (mimeType == KisDocument::nativeFormatMimeType()
-                       || mimeType == "image/openraster") {
-
-                    QScopedPointer<KoStore> store(KoStore::createStore(recentFileUrlPath, KoStore::Read));
-                    if (store) {
-                        QString thumbnailpath;
-                        if (store->hasFile(QString("Thumbnails/thumbnail.png"))){
-                            thumbnailpath = QString("Thumbnails/thumbnail.png");
-                        } else if (store->hasFile(QString("preview.png"))) {
-                            thumbnailpath = QString("preview.png");
-                        }
-                        if (!thumbnailpath.isEmpty()) {
-                            if (store->open(thumbnailpath)) {
-
-                                QByteArray bytes = store->read(store->size());
-                                store->close();
-                                QImage img;
-                                img.loadFromData(bytes);
-                                img.setDevicePixelRatio(devicePixelRatioF());
-                                recentItem->setIcon(QIcon(QPixmap::fromImage(img)));
-                            }
-                        }
-                    }
-                    else {
-                        brokenUrls << m_mainWindow->recentFilesUrls().at(i);
-                    }
-                }
-                else if (mimeType == "image/tiff" || mimeType == "image/x-tiff") {
-                    // Workaround for a bug in Qt tiff QImageIO plugin
-                    QScopedPointer<KisDocument> doc;
-                    doc.reset(KisPart::instance()->createDocument());
-                    doc->setFileBatchMode(true);
-                    bool r = doc->openUrl(QUrl::fromLocalFile(recentFileUrlPath), KisDocument::DontAddToRecent);
-                    if (r) {
-                        KisPaintDeviceSP projection = doc->image()->projection();
-                        recentItem->setIcon(QIcon(QPixmap::fromImage(projection->createThumbnail(48, 48, projection->exactBounds()))));
-                    }
-                    else {
-                        brokenUrls << m_mainWindow->recentFilesUrls().at(i);
-                    }
-                }
-                else {
-                    QImage img;
-                    img.setDevicePixelRatio(devicePixelRatioF());
-                    img.load(recentFileUrlPath);
-                    if (!img.isNull()) {
-                        recentItem->setIcon(QIcon(QPixmap::fromImage(img.scaledToWidth(48))));
-                    }
-                    else {
-                        brokenUrls << m_mainWindow->recentFilesUrls().at(i);
-                    }
-                }
-                if (brokenUrls.size() == 0 || brokenUrls.last().toLocalFile() != recentFileUrlPath) {
-                    m_thumbnailMap[recentFileUrlPath] = recentItem->icon();
-                }
+        } else {
+            QIcon icon;
+            bool success = iconCreator.createFileIcon(recentFileUrlPath, icon, devicePixelRatioF());
+            if (success) {
+                recentItem->setIcon(icon);
+                m_thumbnailMap[recentFileUrlPath] = recentItem->icon();
+            } else {
+                brokenUrls << m_mainWindow->recentFilesUrls().at(i);
             }
+
         }
         Q_FOREACH(const QUrl &url, brokenUrls) {
             m_mainWindow->removeRecentUrl(url);
@@ -372,6 +343,60 @@ void KisWelcomePageWidget::populateRecentDocuments()
     recentDocumentsListView->setModel(&m_recentFilesModel);
 }
 
+void KisWelcomePageWidget::slotUpdateVersionMessage()
+{
+
+    alertIcon->setIcon(KisIconUtils::loadIcon("warning"));
+    alertIcon->setVisible(false);
+
+    // find out if we need an update...or if this is a development version:
+    // dev builds contain GIT hash in it and the word git
+    // stable versions do not contain this.
+    if (qApp->applicationVersion().contains("git")) {
+        // Development build
+        QString versionLabelText = QString("<a style=\"color: " +
+                                           blendedColor.name() +
+                                           " \" href=\"https://docs.krita.org/en/untranslatable_pages/triaging_bugs.html?"
+                                           + analyticsString + "dev-build" + "\">")
+                                  .append(i18n("DEV BUILD")).append("</a>");
+
+        versionNotificationLabel->setText(versionLabelText);
+        alertIcon->setVisible(true);
+        versionNotificationLabel->setVisible(true);
+
+    } else if (newsWidget->hasUpdateAvailable()) {
+
+        // build URL for label
+        QString versionLabelText = QString("<a style=\"color: " +
+                                           blendedColor.name() +
+                                           " \" href=\"" +
+                                           newsWidget->versionLink() + "?" +
+                                           analyticsString + "version-update" + "\">")
+                           .append(i18n("New Version Available!")).append("</a>");
+
+        versionNotificationLabel->setVisible(true);
+        versionNotificationLabel->setText(versionLabelText);
+        alertIcon->setVisible(true);
+
+    } else {
+        // no message needed... exit
+        versionNotificationLabel->setVisible(false);
+        return;
+    }
+
+    if (!blendedStyle.isNull()) {
+        versionNotificationLabel->setStyleSheet(blendedStyle);
+    }
+
+}
+
+#ifdef Q_OS_ANDROID
+void KisWelcomePageWidget::slotStartDonationFlow()
+{
+    QAndroidJniObject::callStaticMethod<void>("org/krita/android/DonationHelper", "startBillingFlow", "()V");
+}
+#endif
+
 void KisWelcomePageWidget::dragEnterEvent(QDragEnterEvent *event)
 {
     //qDebug() << "dragEnterEvent formats" << event->mimeData()->formats() << "urls" << event->mimeData()->urls() << "has images" << event->mimeData()->hasImage();
@@ -442,3 +467,13 @@ void KisWelcomePageWidget::slotOpenFileClicked()
     m_mainWindow->slotFileOpen();
 }
 
+#ifdef Q_OS_ANDROID
+extern "C" JNIEXPORT void JNICALL
+Java_org_krita_android_JNIWrappers_donationSuccessful(JNIEnv* /*env*/,
+                                                      jobject /*obj*/,
+                                                      jint    /*n*/)
+{
+    KisWelcomePageWidget::donationLink->hide();
+    KisWelcomePageWidget::donationBannerImage->show();
+}
+#endif
