@@ -223,7 +223,8 @@ void KisFillActionFactory::run(const QString &fillSource, KisViewManager *view)
     }
 
     KisProcessingVisitorSP visitor =
-        new FillProcessingVisitor(QPoint(0, 0), // start position
+        new FillProcessingVisitor(resources->image()->projection(),
+                                  QPoint(0, 0), // start position
                                   selection,
                                   resources,
                                   false, // fast mode
@@ -232,7 +233,7 @@ void KisFillActionFactory::run(const QString &fillSource, KisViewManager *view)
                                   0, // feathering radius
                                   0, // sizemod
                                   80, // threshold,
-                                  false, // unmerged
+                                  false, // use unmerged
                                   useBgColor);
 
     applicator.applyVisitor(visitor,
@@ -393,6 +394,7 @@ void KisCopyMergedActionFactory::run(KisViewManager *view)
 void KisInvertSelectionOperation::runFromXML(KisViewManager* view, const KisOperationConfiguration& config)
 {
     KisSelectionFilter* filter = new KisInvertSelectionFilter();
+
     runFilter(filter, view, config);
 }
 
@@ -543,8 +545,8 @@ void KisStrokeSelectionActionFactory::run(KisViewManager *view, StrokeSelectionO
     KisNodeSP currentNode = view->canvasResourceProvider()->resourceManager()->resource(KisCanvasResourceProvider::CurrentKritaNode).value<KisNodeWSP>();
     if (!currentNode->inherits("KisShapeLayer") && currentNode->paintDevice()) {
         KoCanvasResourceProvider * rManager = view->canvasResourceProvider()->resourceManager();
-        KisPainter::StrokeStyle strokeStyle =  KisPainter::StrokeStyleBrush;
-        KisPainter::FillStyle fillStyle =  params.fillStyle();
+        KisToolShapeUtils::StrokeStyle strokeStyle =  KisToolShapeUtils::StrokeStyleForeground;
+        KisToolShapeUtils::FillStyle fillStyle = params.fillStyle();
 
         KisFigurePaintingToolHelper helper(kundo2_i18n("Draw Polyline"),
                                        image,
@@ -557,7 +559,7 @@ void KisStrokeSelectionActionFactory::run(KisViewManager *view, StrokeSelectionO
         QPen pen(Qt::red, size);
         pen.setJoinStyle(Qt::RoundJoin);
 
-        if (fillStyle != KisPainter::FillStyleNone) {
+        if (fillStyle != KisToolShapeUtils::FillStyleNone) {
             helper.paintPainterPathQPenFill(outline, pen, params.fillColor);
         }
         else {
@@ -602,8 +604,8 @@ void KisStrokeBrushSelectionActionFactory::run(KisViewManager *view, StrokeSelec
     {
         KoCanvasResourceProvider * rManager = view->canvasResourceProvider()->resourceManager();
         QPainterPath outline = pixelSelection->outlineCache();
-        KisPainter::StrokeStyle strokeStyle =  KisPainter::StrokeStyleBrush;
-        KisPainter::FillStyle fillStyle =  KisPainter::FillStyleNone;
+        KisToolShapeUtils::StrokeStyle strokeStyle =  KisToolShapeUtils::StrokeStyleForeground;
+        KisToolShapeUtils::FillStyle fillStyle =  KisToolShapeUtils::FillStyleNone;
         KoColor color = params.color;
 
         KisFigurePaintingToolHelper helper(kundo2_i18n("Draw Polyline"),
