@@ -76,6 +76,11 @@ char KisPaintingAssistantHandle::handleType() const
     return d->handle_type;
 }
 
+KisPaintingAssistant *KisPaintingAssistantHandle::chiefAssistant() const
+{
+    return !d->assistants.isEmpty() ? d->assistants.first() : 0;
+}
+
 KisPaintingAssistantHandle::~KisPaintingAssistantHandle()
 {
     Q_ASSERT(d->assistants.empty());
@@ -436,6 +441,23 @@ bool KisPaintingAssistant::isAssistantComplete() const
     return true;
 }
 
+void KisPaintingAssistant::transform(const QTransform &transform)
+{
+    Q_FOREACH(KisPaintingAssistantHandleSP handle, handles()) {
+        if (handle->chiefAssistant() != this) continue;
+
+        *handle = transform.map(*handle);
+    }
+
+    Q_FOREACH(KisPaintingAssistantHandleSP handle, sideHandles()) {
+        if (handle->chiefAssistant() != this) continue;
+
+        *handle = transform.map(*handle);
+    }
+
+    uncache();
+}
+
 QByteArray KisPaintingAssistant::saveXml(QMap<KisPaintingAssistantHandleSP, int> &handleMap)
 {
     QByteArray data;
@@ -618,14 +640,22 @@ void KisPaintingAssistant::findPerspectiveAssistantHandleLocation() {
              sort handles on the basis of X-coordinate
              */
             while(hHole > 0 && hHandlesList.at(hHole -1).data()->x() > handle.data()->x()) {
-                hHandlesList.swap(hHole-1, hHole);
+#if QT_VERSION >= QT_VERSION_CHECK(5,13,0)
+                hHandlesList.swapItemsAt(hHole - 1, hHole);
+#else
+                hHandlesList.swap(hHole - 1, hHole);
+#endif
                 hHole = hHole - 1;
             }
             /*
              sort handles on the basis of Y-coordinate
              */
             while(vHole > 0 && vHandlesList.at(vHole -1).data()->y() > handle.data()->y()) {
+#if QT_VERSION >= QT_VERSION_CHECK(5,13,0)
+                vHandlesList.swapItemsAt(vHole-1, vHole);
+#else
                 vHandlesList.swap(vHole-1, vHole);
+#endif
                 vHole = vHole - 1;
             }
         }
@@ -869,6 +899,16 @@ double KisPaintingAssistant::norm2(const QPointF& p)
     return p.x() * p.x() + p.y() * p.y();
 }
 
+QList<KisPaintingAssistantSP> KisPaintingAssistant::cloneAssistantList(const QList<KisPaintingAssistantSP> &list)
+{
+    QMap<KisPaintingAssistantHandleSP, KisPaintingAssistantHandleSP> handleMap;
+    QList<KisPaintingAssistantSP> clonedList;
+    for (auto i = list.begin(); i != list.end(); ++i) {
+        clonedList << (*i)->clone(handleMap);
+    }
+    return clonedList;
+}
+
 
 
 /*
