@@ -53,7 +53,7 @@ struct KisFilterStrokeStrategy::Private {
     }
 
     KisFilterSP filter;
-    KisSafeFilterConfigurationSP filterConfig;
+    KisFilterConfigurationSP filterConfig;
     KisNodeSP node;
     KisUpdatesFacade *updatesFacade;
 
@@ -68,12 +68,12 @@ struct KisFilterStrokeStrategy::Private {
 
 
 KisFilterStrokeStrategy::KisFilterStrokeStrategy(KisFilterSP filter,
-                                                 KisSafeFilterConfigurationSP filterConfig,
+                                                 KisFilterConfigurationSP filterConfig,
                                                  KisResourcesSnapshotSP resources)
-    : KisPainterBasedStrokeStrategy("FILTER_STROKE",
+    : KisPainterBasedStrokeStrategy(QLatin1String("FILTER_STROKE"),
                                     kundo2_i18n("Filter \"%1\"", filter->name()),
                                     resources,
-                                    QVector<PainterInfo*>(),false),
+                                    QVector<KisFreehandStrokeInfo*>(),false),
       m_d(new Private())
 {
     m_d->filter = filter;
@@ -112,7 +112,7 @@ void KisFilterStrokeStrategy::initStrokeCallback()
 
     if (activeSelection() ||
         (dev->colorSpace() != dev->compositionSourceColorSpace() &&
-         !(dev->colorSpace() == dev->compositionSourceColorSpace()))) {
+         *dev->colorSpace() != *dev->compositionSourceColorSpace())) {
 
         m_d->filterDevice = dev->createCompositionSourceDevice(dev);
         m_d->secondaryTransaction = new KisTransaction(m_d->filterDevice);
@@ -166,17 +166,7 @@ void KisFilterStrokeStrategy::cancelStrokeCallback()
     delete m_d->secondaryTransaction;
     m_d->filterDevice = 0;
 
-    KisProjectionUpdatesFilterSP prevUpdatesFilter;
-
     if (m_d->cancelSilently) {
-        /**
-         * TODO: Projection updates filter is not recursive, please
-         *       redesign this part
-         */
-        prevUpdatesFilter = m_d->updatesFacade->projectionUpdatesFilter();
-        if (prevUpdatesFilter) {
-            m_d->updatesFacade->setProjectionUpdatesFilter(KisProjectionUpdatesFilterSP());
-        }
         m_d->updatesFacade->disableDirtyRequests();
     }
 
@@ -184,10 +174,6 @@ void KisFilterStrokeStrategy::cancelStrokeCallback()
 
     if (m_d->cancelSilently) {
         m_d->updatesFacade->enableDirtyRequests();
-        if (prevUpdatesFilter) {
-            m_d->updatesFacade->setProjectionUpdatesFilter(prevUpdatesFilter);
-            prevUpdatesFilter.clear();
-        }
     }
 }
 
@@ -204,6 +190,5 @@ KisStrokeStrategy* KisFilterStrokeStrategy::createLodClone(int levelOfDetail)
     if (!m_d->filter->supportsLevelOfDetail(m_d->filterConfig.data(), levelOfDetail)) return 0;
 
     KisFilterStrokeStrategy *clone = new KisFilterStrokeStrategy(*this, levelOfDetail);
-    clone->setUndoEnabled(false);
     return clone;
 }
