@@ -127,10 +127,16 @@ QString getInstallationPrefix() {
     appdir.setPath(correctedPath);
     appdir.cdUp();
     return appdir.canonicalPath();
+#else
+#ifdef Q_OS_ANDROID
+    // qApp->applicationDirPath() isn't writable and android system won't allow
+    // any files other than libraries
+    return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/";
 #else
     return qApp->applicationDirPath() + "/../";
 #endif
 #endif
+#endif
 }
 
 class Q_DECL_HIDDEN KoResourcePaths::Private {
