@@ -44,6 +44,8 @@ if test -z ${BUILDROOT}; then
     exit
 fi
 
+BUILDROOT="${BUILDROOT%/}"
+
 # print status messages
 print_msg() {
     printf "\e[32m${1}\e[0m\n" "${@:2}"
@@ -99,7 +101,7 @@ print_usage () {
 \t\t\t script will attempt to get password from keychain, if fails provide one with
 \t\t\t the -notarize-pass option: To add a password run 
 
-\t\t\t   security add-generic-password -a \"AC_USERNAME\" -w <secret_password> -s \"KRITA_AC_PASS\"
+\t\t\t   security add-generic-password -a \"AC_USERNAME\" -w <secret_password> -s \"AC_PASSWORD\"
 
     -notarize-pass \t If given, the Apple account password. Otherwise an attempt will be macdeployqt_exists
 \t\t\t to get the password from keychain using the account given in <notarize-ac> option.
@@ -201,11 +203,23 @@ else
     ### NOTARIZATION
 
     if [[ -n "${NOTARIZE_ACC}" ]]; then
-        security find-generic-password -s "KRITA_AC_PASS" > /dev/null 2>&1
-        if [[ ${?} -eq 0 || -n "${NOTARIZE_PASS}" ]]; then
+
+        ASC_PROVIDER_OP=""
+        if [[ -n "${ASC_PROVIDER}" ]]; then
+            ASC_PROVIDER_OP="--asc-provider ${ASC_PROVIDER}"
+        fi
+
+        if [[ -z "${NOTARIZE_PASS}" ]]; then
+            NOTARIZE_PASS="@keychain:AC_PASSWORD"
+        fi
+
+        # check if we can perform notarization
+        xcrun altool --notarization-history 0 --username "${NOTARIZE_ACC}" --password "${NOTARIZE_PASS}" ${ASC_PROVIDER_OP} 1> /dev/null
+
+        if [[ ${?} -eq 0 ]]; then
             NOTARIZE="true"
         else
-            echo "No password given for notarization or KRITA_AC_PASS missig in keychain"
+            echo "No password given for notarization or AC_PASSWORD missig in keychain"
         fi
     fi
 fi
@@ -287,7 +301,7 @@ find_needed_libs () {
                 if [[ "${lib##*/}" = "${libFile##*/}" ]]; then
                     install_name_tool -id ${lib##*/} "${libFile}"
                 else
-                    install_name_tool -change ${lib} "@rpath/${lib##*${BUILDROOT}/i/lib/}" "${libFile}"
+                    install_name_tool -change ${lib} "@rpath/${lib##*/i/lib/}" "${libFile}"
                     local libs_used=$(add_lib_to_list "${lib}" "${libs_used}")
                 fi
             fi
@@ -360,16 +374,10 @@ strip_python_dmginstall() {
     rm -rf Python.app
 }
 
-# Some libraries require r_path to be removed
-# we must not apply delete rpath globally
-delete_install_rpath() {
-    xargs -P4 -I FILE install_name_tool -delete_rpath "${BUILDROOT}/i/lib" FILE 2> "${BUILDROOT}/deploy_error.log"
-}
-
 # Remove any missing rpath poiting to BUILDROOT
 libs_clean_rpath () {
     for libFile in ${@}; do
-        rpath=$(otool -l "${libFile}" | grep "${BUILDROOT}/i/lib" | awk '{$1=$1;print $2}')
+        rpath=$(otool -l "${libFile}" | grep "path ${BUILDROOT}" | awk '{$1=$1;print $2}')
         if [[ -n "${rpath}" ]]; then
             echo "removed rpath _${rpath}_ from ${libFile}"
             install_name_tool -delete_rpath "${rpath}" "${libFile}"
@@ -377,6 +385,12 @@ libs_clean_rpath () {
     done
 }
 
+# Multhread version
+# of libs_clean_rpath, but makes assumptions
+delete_install_rpath() {
+    xargs -P4 -I FILE install_name_tool -delete_rpath "${BUILDROOT}/i/lib" FILE 2> "${BUILDROOT}/deploy_error.log"
+}
+
 fix_python_framework() {
     # Fix python.framework rpath and slims down installation
     PythonFrameworkBase="${KRITA_DMG}/krita.app/Contents/Frameworks/Python.framework"
@@ -399,7 +413,7 @@ fix_python_framework() {
     install_name_tool -add_rpath @executable_path/../../../../ "${PythonFrameworkBase}/Versions/Current/bin/python${PY_VERSION}m"
 
     # Fix rpaths from Python.Framework
-    find ${PythonFrameworkBase} -type f -perm 755 | delete_install_rpath
+    find "${PythonFrameworkBase}" -type f -perm 755 | delete_install_rpath
     find "${PythonFrameworkBase}/Versions/Current/site-packages/PyQt5" -type f -name "*.so" | delete_install_rpath
 }
 
@@ -414,7 +428,7 @@ macdeployqt_exists() {
         printf "Not Found!\n"
         printf "Attempting to install macdeployqt\n"
 
-        cd ${BUILDROOT}/depbuild/ext_qt/ext_qt-prefix/src/ext_qt/qttools/src
+        cd "${BUILDROOT}/depbuild/ext_qt/ext_qt-prefix/src/ext_qt/qttools/src"
         make sub-macdeployqt-all
         make sub-macdeployqt-install_subtargets
         make install
@@ -568,6 +582,7 @@ krita_deploy () {
 
     printf "clean any left over rpath\n"
     libs_clean_rpath $(find "${KRITA_DMG}/krita.app/Contents" -type f -perm 755 -or -name "*.dylib" -or -name "*.so")
+#    libs_clean_rpath $(find "${KRITA_DMG}/krita.app/Contents/" -type f -name "lib*")
 
     echo "Done!"
 
@@ -619,15 +634,6 @@ notarize_build() {
     if [[ ${NOTARIZE} = "true" ]]; then
         printf "performing notarization of %s\n" "${2}"
         cd "${NOT_SRC_DIR}"
-        
-        if [[ -z "${NOTARIZE_PASS}" ]]; then
-            NOTARIZE_PASS="@keychain:KRITA_AC_PASS"
-        fi
-
-        ASC_PROVIDER_OP=""
-        if [[ -n "${ASC_PROVIDER}" ]]; then
-            ASC_PROVIDER_OP="--asc-provider ${ASC_PROVIDER}"
-        fi
 
         ditto -c -k --sequesterRsrc --keepParent "${NOT_SRC_FILE}" "${BUILDROOT}/tmp_notarize/${NOT_SRC_FILE}.zip"
 
@@ -647,7 +653,7 @@ notarize_build() {
         local uuid="$(grep 'RequestUUID' <<< ${altoolResponse} | awk '{ print $NF }')"
         echo "RequestUUID = ${uuid}" # Display identifier string
 
-        waiting_fixed "Waiting to retrieve notarize status" 30
+        waiting_fixed "Waiting to retrieve notarize status" 60
 
         while true ; do
             fullstatus=$(xcrun altool --notarization-info "${uuid}" --username "${NOTARIZE_ACC}" --password "${NOTARIZE_PASS}" ${ASC_PROVIDER_OP} 2>&1)  # get the status
@@ -659,7 +665,7 @@ notarize_build() {
                 print_msg "Notarization success!"
                 break
             elif [[ "${notarize_status}" = "in" ]]; then
-                waiting_fixed "Notarization still in progress, sleeping for 20 seconds and trying again" 20
+                waiting_fixed "Notarization still in progress, sleeping for 60 seconds and trying again" 60
             else
                 echo "Notarization failed! full status below"
                 echo "${fullstatus}"
