@@ -25,6 +25,7 @@
 #include <klocalizedstring.h>
 
 #include <QPainter>
+#include <QPainterPath>
 #include <QLinearGradient>
 #include <QTransform>
 
@@ -203,7 +204,7 @@ QRect FisheyePointAssistant::boundingRect() const
     }
 }
 
-QPointF FisheyePointAssistant::buttonPosition() const
+QPointF FisheyePointAssistant::getEditorPosition() const
 {
     return (*handles()[0] + *handles()[1]) * 0.5;
 }
