@@ -43,7 +43,7 @@ public:
         return KoID("edge detection", i18n("Edge Detection"));
     }
 
-    KisFilterConfigurationSP factoryConfiguration() const override;
+    KisFilterConfigurationSP defaultConfiguration() const override;
 public:
     KisConfigWidget * createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP dev, bool useForMasks) const override;
     QRect neededRect(const QRect & rect, const KisFilterConfigurationSP _config, int lod) const override;
