@@ -180,7 +180,7 @@ void KisKraSaverTest::testRoundTripFillLayerColor()
     Q_ASSERT(generator);
 
     // warning: we pass null paint device to the default constructed value
-    KisFilterConfigurationSP config = generator->factoryConfiguration();
+    KisFilterConfigurationSP config = generator->defaultConfiguration();
     Q_ASSERT(config);
 
     QVariant v;
@@ -196,7 +196,7 @@ void KisKraSaverTest::testRoundTripFillLayerPattern()
     QVERIFY(generator);
 
     // warning: we pass null paint device to the default constructed value
-    KisFilterConfigurationSP config = generator->factoryConfiguration();
+    KisFilterConfigurationSP config = generator->defaultConfiguration();
     QVERIFY(config);
 
     QVariant v;
